@@ -1,16 +1,278 @@
-import math
+import time
+from abc import abstractmethod
 from collections import Counter
-
-import torch
 from typing import List, Union
 
+import torch
 from torch.nn.parameter import Parameter
 
 import flair
 from flair.data import Sentence
-from flair.embeddings import TokenEmbeddings, FlairEmbeddings, StackedEmbeddings, DocumentEmbeddings
+from flair.embeddings import (
+    DocumentEmbeddings,
+    FlairEmbeddings,
+    StackedEmbeddings,
+    TokenEmbeddings,
+)
 from flair.nn import LockedDropout, WordDropout
-import time
+
+
+class TransformerWordEmbeddings(TokenEmbeddings):
+    def __init__(
+        self,
+        model: str = "bert-base-uncased",
+        layers: str = "-1,-2,-3,-4",
+        pooling_operation: str = "first",
+        batch_size: int = 1,
+        use_scalar_mix: bool = False,
+        fine_tune: bool = False,
+    ):
+        """
+        Bidirectional transformer embeddings of words from various transformer architectures.
+        :param model: name of transformer model (see https://huggingface.co/transformers/pretrained_models.html for
+        options)
+        :param layers: string indicating which layers to take for embedding (-1 is topmost layer)
+        :param pooling_operation: how to get from token piece embeddings to token embedding. Either take the first
+        subtoken ('first'), the last subtoken ('last'), both first and last ('first_last') or a mean over all ('mean')
+        :param batch_size: How many sentence to push through transformer at once. Set to 1 by default since transformer
+        models tend to be huge.
+        :param use_scalar_mix: If True, uses a scalar mix of layers as embedding
+        :param fine_tune: If True, allows transformers to be fine-tuned during training
+        """
+        super().__init__()
+
+        # load tokenizer and transformer model
+        self.tokenizer = AutoTokenizer.from_pretrained(model)
+        config = AutoConfig.from_pretrained(model, output_hidden_states=True)
+        self.model = AutoModel.from_pretrained(model, config=config)
+
+        # model name
+        self.name = "transformer-word-" + str(model)
+
+        # when initializing, embeddings are in eval mode by default
+        self.model.eval()
+        self.model.to(flair.device)
+
+        # embedding parameters
+        if layers == "all":
+            # send mini-token through to check how many layers the model has
+            hidden_states = self.model(torch.tensor([1], device=flair.device).unsqueeze(0))[-1]
+            self.layer_indexes = [int(x) for x in range(len(hidden_states))]
+        else:
+            self.layer_indexes = [int(x) for x in layers.split(",")]
+        self.mix = ScalarMix(mixture_size=len(self.layer_indexes), trainable=False)
+        self.pooling_operation = pooling_operation
+        self.use_scalar_mix = use_scalar_mix
+        self.fine_tune = fine_tune
+        self.static_embeddings = not self.fine_tune
+        self.batch_size = batch_size
+
+        self.special_tokens = []
+        self.special_tokens.append(self.tokenizer.bos_token)
+        self.special_tokens.append(self.tokenizer.cls_token)
+
+        # most models have an intial BOS token, except for XLNet, T5 and GPT2
+        self.begin_offset = 1
+        if type(self.tokenizer) == XLNetTokenizer:
+            self.begin_offset = 0
+        if type(self.tokenizer) == T5Tokenizer:
+            self.begin_offset = 0
+        if type(self.tokenizer) == GPT2Tokenizer:
+            self.begin_offset = 0
+
+    def _add_embeddings_internal(self, sentences: List[Sentence]) -> List[Sentence]:
+        """Add embeddings to all words in a list of sentences."""
+
+        # split into micro batches of size self.batch_size before pushing through transformer
+        sentence_batches = [
+            sentences[i * self.batch_size : (i + 1) * self.batch_size]
+            for i in range((len(sentences) + self.batch_size - 1) // self.batch_size)
+        ]
+
+        # embed each micro-batch
+        for batch in sentence_batches:
+            self._add_embeddings_to_sentences(batch)
+
+        return sentences
+
+    def _add_embeddings_to_sentences(self, sentences: List[Sentence]):
+        """Match subtokenization to Flair tokenization and extract embeddings from transformers for each token."""
+
+        # first, subtokenize each sentence and find out into how many subtokens each token was divided
+        subtokenized_sentences = []
+        subtokenized_sentences_token_lengths = []
+
+        for sentence in sentences:
+
+            tokenized_string = sentence.to_tokenized_string()
+
+            # method 1: subtokenize sentence
+            # subtokenized_sentence = self.tokenizer.encode(tokenized_string, add_special_tokens=True)
+
+            # method 2:
+            ids = self.tokenizer.encode(tokenized_string, add_special_tokens=False)
+            subtokenized_sentence = self.tokenizer.build_inputs_with_special_tokens(ids)
+
+            subtokenized_sentences.append(torch.tensor(subtokenized_sentence, dtype=torch.long))
+            subtokens = self.tokenizer.convert_ids_to_tokens(subtokenized_sentence)
+
+            word_iterator = iter(sentence)
+            token = next(word_iterator)
+            token_text = token.text.lower()
+
+            token_subtoken_lengths = []
+            reconstructed_token = ""
+            subtoken_count = 0
+
+            # iterate over subtokens and reconstruct tokens
+            for subtoken_id, subtoken in enumerate(subtokens):
+
+                subtoken_count += 1
+
+                # remove special markup
+                subtoken = re.sub("^Ġ", "", subtoken)  # RoBERTa models
+                subtoken = re.sub("^##", "", subtoken)  # BERT models
+                subtoken = re.sub("^▁", "", subtoken)  # XLNet models
+                subtoken = re.sub("</w>$", "", subtoken)  # XLM models
+
+                # append subtoken to reconstruct token
+                reconstructed_token = reconstructed_token + subtoken
+
+                # check if reconstructed token is special begin token ([CLS] or similar)
+                if reconstructed_token in self.special_tokens and subtoken_id == 0:
+                    reconstructed_token = ""
+                    subtoken_count = 0
+
+                # special handling for UNK subtokens
+                if self.tokenizer.unk_token and self.tokenizer.unk_token in reconstructed_token:
+                    pieces = self.tokenizer.convert_ids_to_tokens(
+                        self.tokenizer.encode(token.text, add_special_tokens=False)
+                    )
+                    token_text = ""
+                    for piece in pieces:
+                        # remove special markup
+                        piece = re.sub("^Ġ", "", piece)  # RoBERTa models
+                        piece = re.sub("^##", "", piece)  # BERT models
+                        piece = re.sub("^▁", "", piece)  # XLNet models
+                        piece = re.sub("</w>$", "", piece)  # XLM models
+                        token_text += piece
+                    token_text = token_text.lower()
+
+                # check if reconstructed token is the same as current token
+                if reconstructed_token.lower() == token_text:
+
+                    # if so, add subtoken count
+                    token_subtoken_lengths.append(subtoken_count)
+
+                    # reset subtoken count and reconstructed token
+                    reconstructed_token = ""
+                    subtoken_count = 0
+
+                    # break from loop if all tokens are accounted for
+                    if len(token_subtoken_lengths) < len(sentence):
+                        token = next(word_iterator)
+                        token_text = token.text.lower()
+                    else:
+                        break
+
+            subtokenized_sentences_token_lengths.append(token_subtoken_lengths)
+
+        # find longest sentence in batch
+        longest_sequence_in_batch: int = len(max(subtokenized_sentences, key=len))
+
+        # initialize batch tensors and mask
+        input_ids = torch.zeros(
+            [len(sentences), longest_sequence_in_batch],
+            dtype=torch.long,
+            device=flair.device,
+        )
+        mask = torch.zeros(
+            [len(sentences), longest_sequence_in_batch],
+            dtype=torch.long,
+            device=flair.device,
+        )
+        for s_id, sentence in enumerate(subtokenized_sentences):
+            sequence_length = len(sentence)
+            input_ids[s_id][:sequence_length] = sentence
+            mask[s_id][:sequence_length] = torch.ones(sequence_length)
+
+        # put encoded batch through transformer model to get all hidden states of all encoder layers
+        hidden_states = self.model(input_ids, attention_mask=mask)[-1]
+
+        # gradients are enabled if fine-tuning is enabled
+        gradient_context = torch.enable_grad() if (self.fine_tune and self.training) else torch.no_grad()
+
+        with gradient_context:
+
+            # iterate over all subtokenized sentences
+            for sentence_idx, (sentence, subtoken_lengths) in enumerate(
+                zip(sentences, subtokenized_sentences_token_lengths)
+            ):
+
+                subword_start_idx = self.begin_offset
+
+                # for each token, get embedding
+                for token_idx, (token, number_of_subtokens) in enumerate(zip(sentence, subtoken_lengths)):
+
+                    subword_end_idx = subword_start_idx + number_of_subtokens
+
+                    subtoken_embeddings: List[torch.FloatTensor] = []
+
+                    # get states from all selected layers, aggregate with pooling operation
+                    for layer in self.layer_indexes:
+                        current_embeddings = hidden_states[layer][sentence_idx][subword_start_idx:subword_end_idx]
+
+                        if self.pooling_operation == "first":
+                            final_embedding: torch.FloatTensor = current_embeddings[0]
+
+                        if self.pooling_operation == "last":
+                            final_embedding: torch.FloatTensor = current_embeddings[-1]
+
+                        if self.pooling_operation == "first_last":
+                            final_embedding: torch.Tensor = torch.cat([current_embeddings[0], current_embeddings[-1]])
+
+                        if self.pooling_operation == "mean":
+                            all_embeddings: List[torch.FloatTensor] = [
+                                embedding.unsqueeze(0) for embedding in current_embeddings
+                            ]
+                            final_embedding: torch.Tensor = torch.mean(torch.cat(all_embeddings, dim=0), dim=0)
+
+                        subtoken_embeddings.append(final_embedding)
+
+                    # use scalar mix of embeddings if so selected
+                    if self.use_scalar_mix:
+                        sm_embeddings = torch.mean(torch.stack(subtoken_embeddings, dim=1), dim=1)
+                        # sm_embeddings = self.mix(subtoken_embeddings)
+
+                        subtoken_embeddings = [sm_embeddings]
+
+                    # set the extracted embedding for the token
+                    token.set_embedding(self.name, torch.cat(subtoken_embeddings))
+
+                    subword_start_idx += number_of_subtokens
+
+    def train(self, mode=True):
+        # if fine-tuning is not enabled (i.e. a "feature-based approach" used), this
+        # module should never be in training mode
+        if not self.fine_tune:
+            pass
+        else:
+            super().train(mode)
+
+    @property
+    @abstractmethod
+    def embedding_length(self) -> int:
+        """Returns the length of the embedding vector."""
+
+        if not self.use_scalar_mix:
+            length = len(self.layer_indexes) * self.model.config.hidden_size
+        else:
+            length = self.model.config.hidden_size
+
+        if self.pooling_operation == "first_last":
+            length *= 2
+
+        return length
 
 
 class Stopwatch(object):
@@ -62,8 +324,7 @@ class Stopwatch(object):
         This is used for checking how much time has elapsed while the timer is
         still running.
         """
-        assert not self.stop_time, \
-            "Can't check `time_elapsed` on an ended `Stopwatch`."
+        assert not self.stop_time, "Can't check `time_elapsed` on an ended `Stopwatch`."
         return time.time() - self.start_time
 
     @property
@@ -92,17 +353,17 @@ class Stopwatch(object):
 
 class DocumentRNNEmbeddingsFast(DocumentEmbeddings):
     def __init__(
-            self,
-            embeddings: List[TokenEmbeddings],
-            hidden_size=128,
-            rnn_layers=1,
-            reproject_words: bool = True,
-            reproject_words_dimension: int = None,
-            bidirectional: bool = False,
-            dropout: float = 0.5,
-            word_dropout: float = 0.0,
-            locked_dropout: float = 0.0,
-            rnn_type="GRU",
+        self,
+        embeddings: List[TokenEmbeddings],
+        hidden_size=128,
+        rnn_layers=1,
+        reproject_words: bool = True,
+        reproject_words_dimension: int = None,
+        bidirectional: bool = False,
+        dropout: float = 0.5,
+        word_dropout: float = 0.0,
+        locked_dropout: float = 0.0,
+        rnn_type="GRU",
     ):
         """The constructor takes a list of embeddings to be combined.
         :param embeddings: a list of token embeddings
@@ -139,9 +400,7 @@ class DocumentRNNEmbeddingsFast(DocumentEmbeddings):
         if self.reproject_words and reproject_words_dimension is not None:
             self.embeddings_dimension = reproject_words_dimension
 
-        self.word_reprojection_map = torch.nn.Linear(
-            self.length_of_all_token_embeddings, self.embeddings_dimension
-        )
+        self.word_reprojection_map = torch.nn.Linear(self.length_of_all_token_embeddings, self.embeddings_dimension)
 
         # bidirectional RNN on top of embedding layer
         if rnn_type == "LSTM":
@@ -181,7 +440,7 @@ class DocumentRNNEmbeddingsFast(DocumentEmbeddings):
 
     def embed(self, sentences: Union[List[Sentence], Sentence]):
         """Add embeddings to all sentences in the given list of sentences. If embeddings are already added, update
-         only if embeddings are non-static."""
+        only if embeddings are non-static."""
 
         if type(sentences) is Sentence:
             sentences = [sentences]
@@ -249,14 +508,14 @@ class DocumentRNNEmbeddingsFast(DocumentEmbeddings):
 
 class MemoryEmbeddingsFast(TokenEmbeddings):
     def __init__(
-            self,
-            contextual_embeddings: TokenEmbeddings,
-            concat_word_embeddings: bool = True,
-            hidden_states: int = 64,
-            reproject: bool = True,
-            max_memory_length: int = 8,
-            dropout: float = 0.5,
-            **kwargs,
+        self,
+        contextual_embeddings: TokenEmbeddings,
+        concat_word_embeddings: bool = True,
+        hidden_states: int = 64,
+        reproject: bool = True,
+        max_memory_length: int = 8,
+        dropout: float = 0.5,
+        **kwargs,
     ):
 
         super().__init__()
@@ -270,9 +529,7 @@ class MemoryEmbeddingsFast(TokenEmbeddings):
 
         # use the character language model embeddings as basis
         if type(contextual_embeddings) is str:
-            self.context_embeddings: FlairEmbeddings = FlairEmbeddings(
-                contextual_embeddings, **kwargs
-            )
+            self.context_embeddings: FlairEmbeddings = FlairEmbeddings(contextual_embeddings, **kwargs)
         else:
             self.context_embeddings: TokenEmbeddings = contextual_embeddings
 
@@ -311,13 +568,13 @@ class MemoryEmbeddingsFast(TokenEmbeddings):
         )
         self.name = self.context_embeddings.name + "-memory"
         #
-        self.all_time = 0.
-        self.prepare_time = 0.
-        self.nn_time = 0.
-        self.add_to_memory_time = 0.
-        self.add_to_words_time = 0.
-        self.pad_time = 0.
-        self.rest_time = 0.
+        self.all_time = 0.0
+        self.prepare_time = 0.0
+        self.nn_time = 0.0
+        self.add_to_memory_time = 0.0
+        self.add_to_words_time = 0.0
+        self.pad_time = 0.0
+        self.rest_time = 0.0
         self.batch_count = 0
 
         self.to(flair.device)
@@ -365,9 +622,7 @@ class MemoryEmbeddingsFast(TokenEmbeddings):
         for sentence in sentences:
             for token in sentence:
                 # add current embedding to the memory
-                local_embedding = token.get_embedding(
-                    self.sub_embedding_names
-                )
+                local_embedding = token.get_embedding(self.sub_embedding_names)
 
                 if token.text not in self.word_history:
                     self.word_history[token.text] = [local_embedding]
@@ -377,9 +632,11 @@ class MemoryEmbeddingsFast(TokenEmbeddings):
                 surface_form_history[token.text] = self.word_history[token.text]
 
         # sort surface forms by longest history
-        surface_forms_sorted_by_memory_length = sorted(surface_form_history,
-                                                       key=lambda k: len(surface_form_history[k]),
-                                                       reverse=True)
+        surface_forms_sorted_by_memory_length = sorted(
+            surface_form_history,
+            key=lambda k: len(surface_form_history[k]),
+            reverse=True,
+        )
         logest_memory_length_in_batch = len(surface_form_history[surface_forms_sorted_by_memory_length[0]])
 
         # all_surface_form_histories = []
@@ -438,11 +695,15 @@ class MemoryEmbeddingsFast(TokenEmbeddings):
             # rest_stopwatch.start()
 
             # truncate surface form history if necessary
-            window = 0 if len(self.word_history[surface_form]) < self.effective_memory_length else len(
-                self.word_history[surface_form]) - self.effective_memory_length
+            window = (
+                0
+                if len(self.word_history[surface_form]) < self.effective_memory_length
+                else len(self.word_history[surface_form]) - self.effective_memory_length
+            )
             if window > 0:
                 self.word_history[surface_form] = self.word_history[surface_form][
-                                                  window:self.effective_memory_length + window]
+                    window : self.effective_memory_length + window
+                ]
 
             # print(surface_form_embedding_history)
 
@@ -461,9 +722,7 @@ class MemoryEmbeddingsFast(TokenEmbeddings):
             if surface_form not in self.state_history:
                 self.state_history[surface_form] = [zeros]
 
-            all_surface_form_initial_hidden.append(
-                self.state_history[surface_form][0]
-            )
+            all_surface_form_initial_hidden.append(self.state_history[surface_form][0])
 
             # rest_stopwatch.stop()
             # self.rest_time += rest_stopwatch.total_run_time
@@ -474,9 +733,7 @@ class MemoryEmbeddingsFast(TokenEmbeddings):
         # print(all_surface_form_tensor)
         # asd
 
-        all_surface_form_initial_hidden = torch.stack(
-            all_surface_form_initial_hidden, 0
-        ).unsqueeze(0)
+        all_surface_form_initial_hidden = torch.stack(all_surface_form_initial_hidden, 0).unsqueeze(0)
 
         # prepare_stopwatch.stop()
         #
@@ -488,9 +745,7 @@ class MemoryEmbeddingsFast(TokenEmbeddings):
 
         # reproject if set
         if self.reproject:
-            all_surface_form_histories = self.reprojection_layer(
-                all_surface_form_histories
-            )
+            all_surface_form_histories = self.reprojection_layer(all_surface_form_histories)
 
         # send through RNN
         # rnn_out, hidden = self.rnn(all_surface_form_histories, all_surface_form_initial_hidden)
@@ -498,9 +753,7 @@ class MemoryEmbeddingsFast(TokenEmbeddings):
 
         packed_output, hidden = self.rnn(packed, all_surface_form_initial_hidden)
 
-        rnn_out, hidden = torch.nn.utils.rnn.pad_packed_sequence(
-            packed_output
-        )
+        rnn_out, hidden = torch.nn.utils.rnn.pad_packed_sequence(packed_output)
 
         # nn_stopwatch.stop()
         #
@@ -509,16 +762,19 @@ class MemoryEmbeddingsFast(TokenEmbeddings):
         # go through each unique surface form and update state history
         for idx, surface_form in enumerate(surface_forms_sorted_by_memory_length):
             # retrieve hidden states for this surface form, and the last initial hidden state
-            hidden_states_of_surface_form = rnn_out[0:lengths[idx], idx]
+            hidden_states_of_surface_form = rnn_out[0 : lengths[idx], idx]
             last_initial_hidden_state = all_surface_form_initial_hidden[0][idx].unsqueeze(0)
 
             # concat both to get new hidden state history
             hidden_state_history = torch.cat([last_initial_hidden_state, hidden_states_of_surface_form], dim=0)
 
             # if history is too long, truncate to window
-            window = 0 if hidden_state_history.size(0) < self.effective_memory_length else hidden_state_history.size(
-                0) - self.effective_memory_length
-            state_history_window = hidden_state_history.detach()[window:self.effective_memory_length + window]
+            window = (
+                0
+                if hidden_state_history.size(0) < self.effective_memory_length
+                else hidden_state_history.size(0) - self.effective_memory_length
+            )
+            state_history_window = hidden_state_history.detach()[window : self.effective_memory_length + window]
 
             # set as new state history of surface form (before: .cpu())
             self.state_history[surface_form] = state_history_window
@@ -583,14 +839,14 @@ class MemoryEmbeddingsFast(TokenEmbeddings):
 
 class MemoryEmbeddings(TokenEmbeddings):
     def __init__(
-            self,
-            contextual_embeddings: TokenEmbeddings,
-            concat_word_embeddings: bool = True,
-            hidden_states: int = 64,
-            reproject: bool = True,
-            max_memory_length: int = 8,
-            dropout: float = 0.5,
-            use_batch_memory_state: bool = False,
+        self,
+        contextual_embeddings: TokenEmbeddings,
+        concat_word_embeddings: bool = True,
+        hidden_states: int = 64,
+        reproject: bool = True,
+        max_memory_length: int = 8,
+        dropout: float = 0.5,
+        use_batch_memory_state: bool = False,
     ):
 
         super().__init__()
@@ -672,9 +928,7 @@ class MemoryEmbeddings(TokenEmbeddings):
         for sentence in sentences:
             for token in sentence:
                 # add current embedding to the memory
-                local_embedding = token.get_embedding(
-                    self.sub_embedding_names
-                )
+                local_embedding = token.get_embedding(self.sub_embedding_names)
 
                 if token.text not in self.word_history:
                     self.word_history[token.text] = [local_embedding]
@@ -684,9 +938,11 @@ class MemoryEmbeddings(TokenEmbeddings):
                 surface_form_history[token.text] = self.word_history[token.text]
 
         # sort surface forms by longest history
-        surface_forms_sorted_by_memory_length = sorted(surface_form_history,
-                                                       key=lambda k: len(surface_form_history[k]),
-                                                       reverse=True)
+        surface_forms_sorted_by_memory_length = sorted(
+            surface_form_history,
+            key=lambda k: len(surface_form_history[k]),
+            reverse=True,
+        )
         logest_memory_length_in_batch = len(surface_form_history[surface_forms_sorted_by_memory_length[0]])
         # print(surface_forms_sorted_by_memory_length)
 
@@ -718,26 +974,26 @@ class MemoryEmbeddings(TokenEmbeddings):
             lengths.append(len(surface_form_embedding_history))
 
             # truncate surface form history if necessary
-            window = 0 if len(self.word_history[surface_form]) < self.effective_memory_length else len(
-                self.word_history[surface_form]) - self.effective_memory_length
+            window = (
+                0
+                if len(self.word_history[surface_form]) < self.effective_memory_length
+                else len(self.word_history[surface_form]) - self.effective_memory_length
+            )
             if window > 0:
                 self.word_history[surface_form] = self.word_history[surface_form][
-                                                  window:self.effective_memory_length + window]
+                    window : self.effective_memory_length + window
+                ]
 
             # initialize first hidden state if necessary
             if surface_form not in self.state_history:
                 self.state_history[surface_form] = zeros
 
-            all_surface_form_initial_hidden.append(
-                self.state_history[surface_form]
-            )
+            all_surface_form_initial_hidden.append(self.state_history[surface_form])
 
         # make batch tensors
         all_surface_form_histories = all_surface_form_tensor
 
-        all_surface_form_initial_hidden = torch.stack(
-            all_surface_form_initial_hidden, 0
-        ).unsqueeze(0)
+        all_surface_form_initial_hidden = torch.stack(all_surface_form_initial_hidden, 0).unsqueeze(0)
 
         # print(all_surface_form_initial_hidden)
         # print(all_surface_form_initial_hidden.size())
@@ -747,18 +1003,14 @@ class MemoryEmbeddings(TokenEmbeddings):
 
         # reproject if set
         if self.reproject:
-            all_surface_form_histories = self.reprojection_layer(
-                all_surface_form_histories
-            )
+            all_surface_form_histories = self.reprojection_layer(all_surface_form_histories)
 
         # send through RNN
         packed = torch.nn.utils.rnn.pack_padded_sequence(all_surface_form_histories, lengths)
 
         packed_output, hidden = self.rnn(packed, all_surface_form_initial_hidden)
 
-        rnn_out, hidden = torch.nn.utils.rnn.pad_packed_sequence(
-            packed_output
-        )
+        rnn_out, hidden = torch.nn.utils.rnn.pad_packed_sequence(packed_output)
 
         # print('rnn out: ')
         # print(rnn_out.size())
@@ -789,11 +1041,22 @@ class MemoryEmbeddings(TokenEmbeddings):
 
                 if self.concat_word_embeddings:
                     import random
+
                     stack = False if self.training and random.randint(1, 100) <= 50 else True
 
-                    embedding = torch.cat([embedding, token.get_embedding(self.sub_embedding_names)]) if stack \
+                    embedding = (
+                        torch.cat([embedding, token.get_embedding(self.sub_embedding_names)])
+                        if stack
                         else torch.cat(
-                        [embedding, torch.zeros(self.embedding_length - self.hidden_states, device=flair.device)])
+                            [
+                                embedding,
+                                torch.zeros(
+                                    self.embedding_length - self.hidden_states,
+                                    device=flair.device,
+                                ),
+                            ]
+                        )
+                    )
 
                 token.set_embedding(self.name, embedding)
 
@@ -817,15 +1080,15 @@ class MemoryEmbeddings(TokenEmbeddings):
 
 class OrderedMemoryEmbeddings(TokenEmbeddings):
     def __init__(
-            self,
-            contextual_embeddings: TokenEmbeddings,
-            concat_word_embeddings: bool = True,
-            hidden_states: int = 64,
-            reproject: bool = False,
-            max_memory_length: int = 8,
-            dropout: float = 0.0,
-            use_batch_memory_state: bool = False,
-            train_initial_hidden_state: bool = True,
+        self,
+        contextual_embeddings: TokenEmbeddings,
+        concat_word_embeddings: bool = True,
+        hidden_states: int = 64,
+        reproject: bool = False,
+        max_memory_length: int = 8,
+        dropout: float = 0.0,
+        use_batch_memory_state: bool = False,
+        train_initial_hidden_state: bool = True,
     ):
 
         super().__init__()
@@ -898,7 +1161,7 @@ class OrderedMemoryEmbeddings(TokenEmbeddings):
             # memory is wiped each time we do a training run
             print("prediction mode no backprop")
             for word in self.last_instance.keys():
-                self.last_state[word] = getattr(self.last_instance[word], 'last_hidden')
+                self.last_state[word] = getattr(self.last_instance[word], "last_hidden")
             self.effective_memory_length = 0
 
             # print(self.last_state)
@@ -919,11 +1182,15 @@ class OrderedMemoryEmbeddings(TokenEmbeddings):
             for sentence in sentences:
                 for token in sentence:
 
-                    if not hasattr(token, 'previous_instance'):
+                    if not hasattr(token, "previous_instance"):
                         if token.text in self.last_instance.keys():
-                            setattr(token, 'previous_instance', self.last_instance[token.text])
+                            setattr(
+                                token,
+                                "previous_instance",
+                                self.last_instance[token.text],
+                            )
                         else:
-                            setattr(token, 'previous_instance', None)
+                            setattr(token, "previous_instance", None)
                     idx += 1
 
                     self.last_instance[token.text] = token
@@ -966,19 +1233,21 @@ class OrderedMemoryEmbeddings(TokenEmbeddings):
                 if self.training:
                     while True:
                         memory = memory.previous_instance
-                        if memory == None: break
-                        if len(word_history[idx]) >= self.effective_memory_length + 1: break
+                        if memory == None:
+                            break
+                        if len(word_history[idx]) >= self.effective_memory_length + 1:
+                            break
 
                         word_history[idx].insert(0, memory)
 
                 for m_id, memory_token in enumerate(word_history[idx]):
                     all_surface_form_tensor[m_id, idx] = memory_token.get_embedding(self.sub_embedding_names)
 
-                all_hidden_tensor[0, idx] = \
-                    self.initial_hidden if token.text not in self.last_state.keys() \
-                    else self.last_state[token.text]
+                all_hidden_tensor[0, idx] = (
+                    self.initial_hidden if token.text not in self.last_state.keys() else self.last_state[token.text]
+                )
 
-                lengths.append(len(word_history[idx]) -1) # length of history only (do not count current token)
+                lengths.append(len(word_history[idx]) - 1)  # length of history only (do not count current token)
 
                 idx += 1
 
@@ -990,9 +1259,7 @@ class OrderedMemoryEmbeddings(TokenEmbeddings):
 
         # reproject if set
         if self.reproject:
-            all_surface_form_histories = self.reprojection_layer(
-                all_surface_form_histories
-            )
+            all_surface_form_histories = self.reprojection_layer(all_surface_form_histories)
 
         # print(all_surface_form_histories.size())
         rnn_out, hidden = self.rnn(all_surface_form_histories, all_hidden_tensor)
@@ -1010,24 +1277,36 @@ class OrderedMemoryEmbeddings(TokenEmbeddings):
                 if last_memory_index >= 0:
                     embedding = rnn_out[last_memory_index, idx]
                 else:
-                    embedding = self.initial_hidden if token.text not in self.last_state.keys() \
-                                else self.last_state[token.text]
+                    embedding = (
+                        self.initial_hidden if token.text not in self.last_state.keys() else self.last_state[token.text]
+                    )
 
                 # remember last hidden state of token
                 last_hidden = rnn_out[updated_memory_index, idx]
 
                 if self.training:
-                    setattr(token, 'last_hidden', last_hidden.clone().detach())
+                    setattr(token, "last_hidden", last_hidden.clone().detach())
                 else:
                     self.last_state[token.text] = last_hidden.clone().detach()
 
                 if self.concat_word_embeddings:
                     import random
+
                     stack = False if self.training and random.randint(1, 100) <= 50 else True
 
-                    embedding = torch.cat([embedding, token.get_embedding(self.sub_embedding_names)]) if stack \
+                    embedding = (
+                        torch.cat([embedding, token.get_embedding(self.sub_embedding_names)])
+                        if stack
                         else torch.cat(
-                        [embedding, torch.zeros(self.embedding_length - self.hidden_states, device=flair.device)])
+                            [
+                                embedding,
+                                torch.zeros(
+                                    self.embedding_length - self.hidden_states,
+                                    device=flair.device,
+                                ),
+                            ]
+                        )
+                    )
 
                 token.set_embedding(self.name, embedding)
 
@@ -1045,7 +1324,7 @@ class OrderedMemoryEmbeddings(TokenEmbeddings):
     def __getstate__(self):
 
         for word in self.last_instance.keys():
-            self.last_state[word] = getattr(self.last_instance[word], 'last_hidden')
+            self.last_state[word] = getattr(self.last_instance[word], "last_hidden")
         self.last_instance = {}
 
         return self.__dict__
