@@ -1,6 +1,6 @@
 import logging
 from pathlib import Path
-from typing import List, Union, Optional, Callable, Dict
+from typing import Callable, Dict, List, Optional, Union
 
 import numpy as np
 import torch
@@ -12,9 +12,9 @@ from torch.utils.data.dataset import Dataset
 from tqdm import tqdm
 
 import flair.nn
-from flair.data import Dictionary, Sentence, Token, Label, space_tokenizer, DataPoint
-from flair.datasets import SentenceDataset, StringDataset, DataLoader
-from flair.embeddings import TokenEmbeddings, StackedEmbeddings
+from flair.data import DataPoint, Dictionary, Label, Sentence, Token, space_tokenizer
+from flair.datasets import DataLoader, SentenceDataset, StringDataset
+from flair.embeddings import StackedEmbeddings, TokenEmbeddings
 from flair.file_utils import cached_path, unzip_file
 from flair.local_rnn import MemoryLSTM
 from flair.training_utils import Metric, Result, store_embeddings
@@ -77,7 +77,7 @@ class MemoryTagger(flair.nn.Model):
         dropout: float = 0.0,
         word_dropout: float = 0.05,
         locked_dropout: float = 0.5,
-        reproject_embeddings: Union[bool,int] = True,
+        reproject_embeddings: Union[bool, int] = True,
         train_initial_hidden_state: bool = False,
         rnn_type: str = "LSTM",
         pickle_module: str = "pickle",
@@ -132,7 +132,7 @@ class MemoryTagger(flair.nn.Model):
         # Initialize the weight tensor
         if loss_weights is not None:
             n_classes = len(self.tag_dictionary)
-            weight_list = [1. for i in range(n_classes)]
+            weight_list = [1.0 for i in range(n_classes)]
             for i, tag in enumerate(self.tag_dictionary.get_items()):
                 if tag in loss_weights.keys():
                     weight_list[i] = loss_weights[tag]
@@ -188,17 +188,11 @@ class MemoryTagger(flair.nn.Model):
         self.linear = torch.nn.Linear(self.rnn.output_size, len(tag_dictionary))
 
         if self.use_crf:
-            self.transitions = torch.nn.Parameter(
-                torch.randn(self.tagset_size, self.tagset_size)
-            )
+            self.transitions = torch.nn.Parameter(torch.randn(self.tagset_size, self.tagset_size))
 
-            self.transitions.detach()[
-                self.tag_dictionary.get_idx_for_item(START_TAG), :
-            ] = -10000
+            self.transitions.detach()[self.tag_dictionary.get_idx_for_item(START_TAG), :] = -10000
 
-            self.transitions.detach()[
-                :, self.tag_dictionary.get_idx_for_item(STOP_TAG)
-            ] = -10000
+            self.transitions.detach()[:, self.tag_dictionary.get_idx_for_item(STOP_TAG)] = -10000
 
         self.to(flair.device)
 
@@ -229,18 +223,10 @@ class MemoryTagger(flair.nn.Model):
 
         rnn_type = "LSTM" if "rnn_type" not in state.keys() else state["rnn_type"]
         use_dropout = 0.0 if "use_dropout" not in state.keys() else state["use_dropout"]
-        use_word_dropout = (
-            0.0 if "use_word_dropout" not in state.keys() else state["use_word_dropout"]
-        )
-        use_locked_dropout = (
-            0.0
-            if "use_locked_dropout" not in state.keys()
-            else state["use_locked_dropout"]
-        )
+        use_word_dropout = 0.0 if "use_word_dropout" not in state.keys() else state["use_word_dropout"]
+        use_locked_dropout = 0.0 if "use_locked_dropout" not in state.keys() else state["use_locked_dropout"]
         train_initial_hidden_state = (
-            False
-            if "train_initial_hidden_state" not in state.keys()
-            else state["train_initial_hidden_state"]
+            False if "train_initial_hidden_state" not in state.keys() else state["train_initial_hidden_state"]
         )
         beta = 1.0 if "beta" not in state.keys() else state["beta"]
         weights = None if "weight_dict" not in state.keys() else state["weight_dict"]
@@ -276,7 +262,7 @@ class MemoryTagger(flair.nn.Model):
         all_tag_prob: bool = False,
         verbose: bool = False,
         label_name: Optional[str] = None,
-        return_loss = False,
+        return_loss=False,
         embedding_storage_mode="none",
     ):
         """
@@ -304,17 +290,11 @@ class MemoryTagger(flair.nn.Model):
                 sentences = [sentences]
 
             # reverse sort all sequences by their length
-            rev_order_len_index = sorted(
-                range(len(sentences)), key=lambda k: len(sentences[k]), reverse=True
-            )
+            rev_order_len_index = sorted(range(len(sentences)), key=lambda k: len(sentences[k]), reverse=True)
 
-            reordered_sentences: List[Union[Sentence, str]] = [
-                sentences[index] for index in rev_order_len_index
-            ]
+            reordered_sentences: List[Union[Sentence, str]] = [sentences[index] for index in rev_order_len_index]
 
-            dataloader = DataLoader(
-                dataset=SentenceDataset(reordered_sentences), batch_size=mini_batch_size
-            )
+            dataloader = DataLoader(dataset=SentenceDataset(reordered_sentences), batch_size=mini_batch_size)
 
             if self.use_crf:
                 transitions = self.transitions.detach().cpu().numpy()
@@ -369,7 +349,7 @@ class MemoryTagger(flair.nn.Model):
     def _requires_span_F1_evaluation(self) -> bool:
         span_F1 = False
         for item in self.tag_dictionary.get_items():
-            if item.startswith('B-'):
+            if item.startswith("B-"):
                 span_F1 = True
         return span_F1
 
@@ -388,11 +368,13 @@ class MemoryTagger(flair.nn.Model):
         for batch in data_loader:
 
             # predict for batch
-            loss = self.predict(batch,
-                                embedding_storage_mode=embedding_storage_mode,
-                                mini_batch_size=mini_batch_size,
-                                label_name='predicted',
-                                return_loss=True)
+            loss = self.predict(
+                batch,
+                embedding_storage_mode=embedding_storage_mode,
+                mini_batch_size=mini_batch_size,
+                label_name="predicted",
+                return_loss=True,
+            )
             eval_loss += loss
             batch_no += 1
 
@@ -424,21 +406,21 @@ class MemoryTagger(flair.nn.Model):
                 if out_path:
                     for token in sentence:
                         # check if in gold spans
-                        gold_tag = 'O'
+                        gold_tag = "O"
                         for span in gold_spans:
                             if token in span:
-                                gold_tag = 'B-' + span.tag if token == span[0] else 'I-' + span.tag
+                                gold_tag = "B-" + span.tag if token == span[0] else "I-" + span.tag
                         tags_gold.append(gold_tag)
 
-                        predicted_tag = 'O'
+                        predicted_tag = "O"
                         # check if in predicted spans
                         for span in predicted_spans:
                             if token in span:
-                                predicted_tag = 'B-' + span.tag if token == span[0] else 'I-' + span.tag
+                                predicted_tag = "B-" + span.tag if token == span[0] else "I-" + span.tag
                         tags_pred.append(predicted_tag)
 
-                        lines.append(f'{token.text} {gold_tag} {predicted_tag}\n')
-                    lines.append('\n')
+                        lines.append(f"{token.text} {gold_tag} {predicted_tag}\n")
+                    lines.append("\n")
 
                 y_true.append(tags_gold)
                 y_pred.append(tags_pred)
@@ -453,7 +435,7 @@ class MemoryTagger(flair.nn.Model):
             "\nResults:"
             f"\n- F1-score (micro) {metric.micro_avg_f_score():.4f}"
             f"\n- F1-score (macro) {metric.macro_avg_f_score():.4f}"
-            '\n\nBy class:'
+            "\n\nBy class:"
         )
 
         for class_name in metric.get_classes():
@@ -505,11 +487,13 @@ class MemoryTagger(flair.nn.Model):
         for batch in data_loader:
 
             # predict for batch
-            loss = self.predict(batch,
-                                embedding_storage_mode=embedding_storage_mode,
-                                mini_batch_size=mini_batch_size,
-                                label_name='predicted',
-                                return_loss=True)
+            loss = self.predict(
+                batch,
+                embedding_storage_mode=embedding_storage_mode,
+                mini_batch_size=mini_batch_size,
+                label_name="predicted",
+                return_loss=True,
+            )
             eval_loss += loss
             batch_no += 1
 
@@ -521,13 +505,13 @@ class MemoryTagger(flair.nn.Model):
                     y_true.append(labels.add_item(gold_tag))
 
                     # add predicted tag
-                    predicted_tag = token.get_tag('predicted').value
+                    predicted_tag = token.get_tag("predicted").value
                     y_pred.append(labels.add_item(predicted_tag))
 
                     # for file output
-                    lines.append(f'{token.text} {gold_tag} {predicted_tag}\n')
+                    lines.append(f"{token.text} {gold_tag} {predicted_tag}\n")
 
-                lines.append('\n')
+                lines.append("\n")
 
         if out_path:
             with open(Path(out_path), "w", encoding="utf-8") as outfile:
@@ -542,11 +526,13 @@ class MemoryTagger(flair.nn.Model):
         target_names = []
         for i in range(len(labels)):
             target_names.append(labels.get_item_for_index(i))
-        classification_report = metrics.classification_report(y_true, y_pred, digits=4, target_names=target_names, zero_division=1)
+        classification_report = metrics.classification_report(
+            y_true, y_pred, digits=4, target_names=target_names, zero_division=1
+        )
 
         # get scores
-        macro_f_score = round(metrics.fbeta_score(y_true, y_pred, beta=self.beta, average='micro'), 4)
-        micro_f_score = round(metrics.fbeta_score(y_true, y_pred, beta=self.beta, average='macro'), 4)
+        macro_f_score = round(metrics.fbeta_score(y_true, y_pred, beta=self.beta, average="micro"), 4)
+        micro_f_score = round(metrics.fbeta_score(y_true, y_pred, beta=self.beta, average="macro"), 4)
         accuracy_score = round(metrics.accuracy_score(y_true, y_pred), 4)
 
         detailed_result = (
@@ -554,7 +540,7 @@ class MemoryTagger(flair.nn.Model):
             f"\n- F-score (micro) {macro_f_score}"
             f"\n- F-score (macro) {micro_f_score}"
             f"\n- Accuracy {accuracy_score}"
-            '\n\nBy class:\n' + classification_report
+            "\n\nBy class:\n" + classification_report
         )
 
         # line for log file
@@ -569,9 +555,7 @@ class MemoryTagger(flair.nn.Model):
         )
         return result, eval_loss
 
-    def forward_loss(
-        self, data_points: Union[List[Sentence], Sentence], sort=True
-    ) -> torch.tensor:
+    def forward_loss(self, data_points: Union[List[Sentence], Sentence], sort=True) -> torch.tensor:
         features = self.forward(data_points)
         return self._calculate_loss(features, data_points)
 
@@ -594,9 +578,7 @@ class MemoryTagger(flair.nn.Model):
         all_embs = list()
         for sentence in sentences:
             t_ids.append([token.text for token in sentence])
-            all_embs += [
-                emb for token in sentence for emb in token.get_each_embedding()
-            ]
+            all_embs += [emb for token in sentence for emb in token.get_each_embedding()]
             nb_padding_tokens = longest_token_sequence_in_batch - len(sentence)
 
             if nb_padding_tokens > 0:
@@ -604,7 +586,11 @@ class MemoryTagger(flair.nn.Model):
                 all_embs.append(t)
 
         sentence_tensor = torch.cat(all_embs).view(
-            [len(sentences), longest_token_sequence_in_batch, self.embeddings.embedding_length]
+            [
+                len(sentences),
+                longest_token_sequence_in_batch,
+                self.embeddings.embedding_length,
+            ]
         )
 
         # --------------------------------------------------------------------
@@ -645,23 +631,17 @@ class MemoryTagger(flair.nn.Model):
 
     def _score_sentence(self, feats, tags, lens_):
 
-        start = torch.tensor(
-            [self.tag_dictionary.get_idx_for_item(START_TAG)], device=flair.device
-        )
+        start = torch.tensor([self.tag_dictionary.get_idx_for_item(START_TAG)], device=flair.device)
         start = start[None, :].repeat(tags.shape[0], 1)
 
-        stop = torch.tensor(
-            [self.tag_dictionary.get_idx_for_item(STOP_TAG)], device=flair.device
-        )
+        stop = torch.tensor([self.tag_dictionary.get_idx_for_item(STOP_TAG)], device=flair.device)
         stop = stop[None, :].repeat(tags.shape[0], 1)
 
         pad_start_tags = torch.cat([start, tags], 1)
         pad_stop_tags = torch.cat([tags, stop], 1)
 
         for i in range(len(lens_)):
-            pad_stop_tags[i, lens_[i] :] = self.tag_dictionary.get_idx_for_item(
-                STOP_TAG
-            )
+            pad_stop_tags[i, lens_[i] :] = self.tag_dictionary.get_idx_for_item(STOP_TAG)
 
         score = torch.FloatTensor(feats.shape[0]).to(flair.device)
 
@@ -669,16 +649,12 @@ class MemoryTagger(flair.nn.Model):
             r = torch.LongTensor(range(lens_[i])).to(flair.device)
 
             score[i] = torch.sum(
-                self.transitions[
-                    pad_stop_tags[i, : lens_[i] + 1], pad_start_tags[i, : lens_[i] + 1]
-                ]
+                self.transitions[pad_stop_tags[i, : lens_[i] + 1], pad_start_tags[i, : lens_[i] + 1]]
             ) + torch.sum(feats[i, r, tags[i, : lens_[i]]])
 
         return score
 
-    def _calculate_loss(
-        self, features: torch.tensor, sentences: List[Sentence]
-    ) -> float:
+    def _calculate_loss(self, features: torch.tensor, sentences: List[Sentence]) -> float:
 
         lengths: List[int] = [len(sentence.tokens) for sentence in sentences]
 
@@ -686,8 +662,7 @@ class MemoryTagger(flair.nn.Model):
         for s_id, sentence in enumerate(sentences):
             # get the tags in this sentence
             tag_idx: List[int] = [
-                self.tag_dictionary.get_idx_for_item(token.get_tag(self.tag_type).value)
-                for token in sentence
+                self.tag_dictionary.get_idx_for_item(token.get_tag(self.tag_type).value) for token in sentence
             ]
             # add tags as tensor
             tag = torch.tensor(tag_idx, device=flair.device)
@@ -706,13 +681,9 @@ class MemoryTagger(flair.nn.Model):
 
         else:
             score = 0
-            for sentence_feats, sentence_tags, sentence_length in zip(
-                features, tag_list, lengths
-            ):
+            for sentence_feats, sentence_tags, sentence_length in zip(features, tag_list, lengths):
                 sentence_feats = sentence_feats[:sentence_length]
-                score += torch.nn.functional.cross_entropy(
-                    sentence_feats, sentence_tags, weight=self.loss_weights
-                )
+                score += torch.nn.functional.cross_entropy(sentence_feats, sentence_tags, weight=self.loss_weights)
             score /= len(features)
             return score
 
@@ -758,19 +729,14 @@ class MemoryTagger(flair.nn.Model):
                 scores = softmax[:length].tolist()
 
             tags.append(
-                [
-                    Label(self.tag_dictionary.get_item_for_index(tag), conf)
-                    for conf, tag in zip(confidences, tag_seq)
-                ]
+                [Label(self.tag_dictionary.get_item_for_index(tag), conf) for conf, tag in zip(confidences, tag_seq)]
             )
 
             if get_all_tags:
                 all_tags.append(
                     [
                         [
-                            Label(
-                                self.tag_dictionary.get_item_for_index(score_id), score
-                            )
+                            Label(self.tag_dictionary.get_item_for_index(score_id), score)
                             for score_id, score in enumerate(score_dist)
                         ]
                         for score_dist in scores
@@ -786,20 +752,14 @@ class MemoryTagger(flair.nn.Model):
         y = np.exp(x_norm)
         return y / y.sum(axis=axis, keepdims=True)
 
-    def _viterbi_decode(
-        self, feats: np.ndarray, transitions: np.ndarray, all_scores: bool
-    ):
+    def _viterbi_decode(self, feats: np.ndarray, transitions: np.ndarray, all_scores: bool):
         id_start = self.tag_dictionary.get_idx_for_item(START_TAG)
         id_stop = self.tag_dictionary.get_idx_for_item(STOP_TAG)
 
         backpointers = np.empty(shape=(feats.shape[0], self.tagset_size), dtype=np.int_)
-        backscores = np.empty(
-            shape=(feats.shape[0], self.tagset_size), dtype=np.float32
-        )
+        backscores = np.empty(shape=(feats.shape[0], self.tagset_size), dtype=np.float32)
 
-        init_vvars = np.expand_dims(
-            np.repeat(-10000.0, self.tagset_size), axis=0
-        ).astype(np.float32)
+        init_vvars = np.expand_dims(np.repeat(-10000.0, self.tagset_size), axis=0).astype(np.float32)
         init_vvars[0][id_start] = 0
 
         forward_var = init_vvars
@@ -837,19 +797,13 @@ class MemoryTagger(flair.nn.Model):
             for index, (tag_id, tag_scores) in enumerate(zip(best_path, all_scores_np)):
                 if type(tag_id) != int and tag_id.item() != tag_scores.argmax():
                     swap_index_score = tag_scores.argmax()
-                    (
-                        all_scores_np[index][tag_id.item()],
-                        all_scores_np[index][swap_index_score],
-                    ) = (
+                    (all_scores_np[index][tag_id.item()], all_scores_np[index][swap_index_score],) = (
                         all_scores_np[index][swap_index_score],
                         all_scores_np[index][tag_id.item()],
                     )
                 elif type(tag_id) == int and tag_id != tag_scores.argmax():
                     swap_index_score = tag_scores.argmax()
-                    (
-                        all_scores_np[index][tag_id],
-                        all_scores_np[index][swap_index_score],
-                    ) = (
+                    (all_scores_np[index][tag_id], all_scores_np[index][swap_index_score],) = (
                         all_scores_np[index][swap_index_score],
                         all_scores_np[index][tag_id],
                     )
@@ -871,9 +825,9 @@ class MemoryTagger(flair.nn.Model):
 
         forward_var[:, 0, :] = init_alphas[None, :].repeat(feats.shape[0], 1)
 
-        transitions = self.transitions.view(
-            1, self.transitions.shape[0], self.transitions.shape[1]
-        ).repeat(feats.shape[0], 1, 1)
+        transitions = self.transitions.view(1, self.transitions.shape[0], self.transitions.shape[1]).repeat(
+            feats.shape[0], 1, 1
+        )
 
         for i in range(feats.shape[1]):
             emit_score = feats[:, i, :]
@@ -881,16 +835,12 @@ class MemoryTagger(flair.nn.Model):
             tag_var = (
                 emit_score[:, :, None].repeat(1, 1, transitions.shape[2])
                 + transitions
-                + forward_var[:, i, :][:, :, None]
-                .repeat(1, 1, transitions.shape[2])
-                .transpose(2, 1)
+                + forward_var[:, i, :][:, :, None].repeat(1, 1, transitions.shape[2]).transpose(2, 1)
             )
 
             max_tag_var, _ = torch.max(tag_var, dim=2)
 
-            tag_var = tag_var - max_tag_var[:, :, None].repeat(
-                1, 1, transitions.shape[2]
-            )
+            tag_var = tag_var - max_tag_var[:, :, None].repeat(1, 1, transitions.shape[2])
 
             agg_ = torch.log(torch.sum(torch.exp(tag_var), dim=2))
 
@@ -901,9 +851,9 @@ class MemoryTagger(flair.nn.Model):
 
         forward_var = forward_var[range(forward_var.shape[0]), lens_, :]
 
-        terminal_var = forward_var + self.transitions[
-            self.tag_dictionary.get_idx_for_item(STOP_TAG)
-        ][None, :].repeat(forward_var.shape[0], 1)
+        terminal_var = forward_var + self.transitions[self.tag_dictionary.get_idx_for_item(STOP_TAG)][None, :].repeat(
+            forward_var.shape[0], 1
+        )
 
         alpha = log_sum_exp_batch(terminal_var)
 
@@ -913,18 +863,14 @@ class MemoryTagger(flair.nn.Model):
     def _filter_empty_sentences(sentences: List[Sentence]) -> List[Sentence]:
         filtered_sentences = [sentence for sentence in sentences if sentence.tokens]
         if len(sentences) != len(filtered_sentences):
-            log.warning(
-                f"Ignore {len(sentences) - len(filtered_sentences)} sentence(s) with no tokens."
-            )
+            log.warning(f"Ignore {len(sentences) - len(filtered_sentences)} sentence(s) with no tokens.")
         return filtered_sentences
 
     @staticmethod
     def _filter_empty_string(texts: List[str]) -> List[str]:
         filtered_texts = [text for text in texts if text]
         if len(texts) != len(filtered_texts):
-            log.warning(
-                f"Ignore {len(texts) - len(filtered_texts)} string(s) with no tokens."
-            )
+            log.warning(f"Ignore {len(texts) - len(filtered_texts)} string(s) with no tokens.")
         return filtered_texts
 
     @staticmethod
@@ -935,9 +881,7 @@ class MemoryTagger(flair.nn.Model):
         aws_resource_path_v04 = "https://s3.eu-central-1.amazonaws.com/alan-nlp/resources/models-v0.4"
         hu_path: str = "https://nlp.informatik.hu-berlin.de/resources/models"
 
-        model_map["ner"] = "/".join(
-            [aws_resource_path_v04, "NER-conll03-english", "en-ner-conll03-v0.4.pt"]
-        )
+        model_map["ner"] = "/".join([aws_resource_path_v04, "NER-conll03-english", "en-ner-conll03-v0.4.pt"])
 
         model_map["ner-fast"] = "/".join(
             [
@@ -973,9 +917,7 @@ class MemoryTagger(flair.nn.Model):
             )
 
         for key in ["ner-multi-fast", "multi-ner-fast"]:
-            model_map[key] = "/".join(
-                [aws_resource_path_v04, "NER-multi-fast", "ner-multi-fast.pt"]
-            )
+            model_map[key] = "/".join([aws_resource_path_v04, "NER-multi-fast", "ner-multi-fast.pt"])
 
         for key in ["ner-multi-fast-learn", "multi-ner-fast-learn"]:
             model_map[key] = "/".join(
@@ -1028,13 +970,9 @@ class MemoryTagger(flair.nn.Model):
             )
 
         for key in ["pos-multi-fast", "multi-pos-fast"]:
-            model_map[key] = "/".join(
-                [aws_resource_path_v04, "UPOS-multi-fast", "pos-multi-fast.pt"]
-            )
+            model_map[key] = "/".join([aws_resource_path_v04, "UPOS-multi-fast", "pos-multi-fast.pt"])
 
-        model_map["frame"] = "/".join(
-            [aws_resource_path_v04, "release-frame-1", "en-frame-ontonotes-v0.4.pt"]
-        )
+        model_map["frame"] = "/".join([aws_resource_path_v04, "release-frame-1", "en-frame-ontonotes-v0.4.pt"])
 
         model_map["frame-fast"] = "/".join(
             [
@@ -1060,17 +998,11 @@ class MemoryTagger(flair.nn.Model):
             ]
         )
 
-        model_map["da-pos"] = "/".join(
-            [aws_resource_path_v04, "POS-danish", "da-pos-v0.1.pt"]
-        )
+        model_map["da-pos"] = "/".join([aws_resource_path_v04, "POS-danish", "da-pos-v0.1.pt"])
 
-        model_map["da-ner"] = "/".join(
-            [aws_resource_path_v04, "NER-danish", "da-ner-v0.1.pt"]
-        )
+        model_map["da-ner"] = "/".join([aws_resource_path_v04, "NER-danish", "da-ner-v0.1.pt"])
 
-        model_map["de-pos"] = "/".join(
-            [hu_path, "release-de-pos-0", "de-pos-ud-hdt-v0.5.pt"]
-        )
+        model_map["de-pos"] = "/".join([hu_path, "release-de-pos-0", "de-pos-ud-hdt-v0.5.pt"])
 
         model_map["de-pos-tweets"] = "/".join(
             [
@@ -1080,29 +1012,21 @@ class MemoryTagger(flair.nn.Model):
             ]
         )
 
-        model_map["de-ner"] = "/".join(
-            [aws_resource_path_v04, "release-de-ner-0", "de-ner-conll03-v0.4.pt"]
-        )
+        model_map["de-ner"] = "/".join([aws_resource_path_v04, "release-de-ner-0", "de-ner-conll03-v0.4.pt"])
 
-        model_map["de-ner-germeval"] = "/".join(
-            [aws_resource_path_v04, "NER-germeval", "de-ner-germeval-0.4.1.pt"]
-        )
+        model_map["de-ner-germeval"] = "/".join([aws_resource_path_v04, "NER-germeval", "de-ner-germeval-0.4.1.pt"])
 
-        model_map["fr-ner"] = "/".join(
-            [aws_resource_path_v04, "release-fr-ner-0", "fr-ner-wikiner-0.4.pt"]
-        )
-        model_map["nl-ner"] = "/".join(
-            [hu_path, "dutch-ner_0", "nl-ner-bert-conll02-v0.5.pt"]
-        )
-        model_map["nl-ner-rnn"] = "/".join(
-            [hu_path, "dutch-ner-flair-0", "nl-ner-conll02-v0.5.pt"]
-        )
-        model_map["ml-pos"] = "https://raw.githubusercontent.com/qburst/models-repository/master/FlairMalayalamModels/malayalam-xpos-model.pt"
-        model_map["ml-upos"] = "https://raw.githubusercontent.com/qburst/models-repository/master/FlairMalayalamModels/malayalam-upos-model.pt"
+        model_map["fr-ner"] = "/".join([aws_resource_path_v04, "release-fr-ner-0", "fr-ner-wikiner-0.4.pt"])
+        model_map["nl-ner"] = "/".join([hu_path, "dutch-ner_0", "nl-ner-bert-conll02-v0.5.pt"])
+        model_map["nl-ner-rnn"] = "/".join([hu_path, "dutch-ner-flair-0", "nl-ner-conll02-v0.5.pt"])
+        model_map[
+            "ml-pos"
+        ] = "https://raw.githubusercontent.com/qburst/models-repository/master/FlairMalayalamModels/malayalam-xpos-model.pt"
+        model_map[
+            "ml-upos"
+        ] = "https://raw.githubusercontent.com/qburst/models-repository/master/FlairMalayalamModels/malayalam-upos-model.pt"
 
-        model_map["keyphrase"] = "/".join(
-            [hu_path, "keyphrase-semeval2017-scibert", "keyphrase-en-scibert.pt"]
-        )
+        model_map["keyphrase"] = "/".join([hu_path, "keyphrase-semeval2017-scibert", "keyphrase-en-scibert.pt"])
 
         cache_dir = Path("models")
         if model_name in model_map:
@@ -1110,32 +1034,56 @@ class MemoryTagger(flair.nn.Model):
 
         # the historical German taggers by the @redewiegergabe project
         if model_name == "de-historic-indirect":
-            model_file = Path(flair.cache_root)  / cache_dir / 'indirect' / 'final-model.pt'
+            model_file = Path(flair.cache_root) / cache_dir / "indirect" / "final-model.pt"
             if not model_file.exists():
-                cached_path('http://www.redewiedergabe.de/models/indirect.zip', cache_dir=cache_dir)
-                unzip_file(Path(flair.cache_root)  / cache_dir / 'indirect.zip', Path(flair.cache_root)  / cache_dir)
-            model_name = str(Path(flair.cache_root)  / cache_dir / 'indirect' / 'final-model.pt')
+                cached_path(
+                    "http://www.redewiedergabe.de/models/indirect.zip",
+                    cache_dir=cache_dir,
+                )
+                unzip_file(
+                    Path(flair.cache_root) / cache_dir / "indirect.zip",
+                    Path(flair.cache_root) / cache_dir,
+                )
+            model_name = str(Path(flair.cache_root) / cache_dir / "indirect" / "final-model.pt")
 
         if model_name == "de-historic-direct":
-            model_file = Path(flair.cache_root)  / cache_dir / 'direct' / 'final-model.pt'
+            model_file = Path(flair.cache_root) / cache_dir / "direct" / "final-model.pt"
             if not model_file.exists():
-                cached_path('http://www.redewiedergabe.de/models/direct.zip', cache_dir=cache_dir)
-                unzip_file(Path(flair.cache_root)  / cache_dir / 'direct.zip', Path(flair.cache_root)  / cache_dir)
-            model_name = str(Path(flair.cache_root)  / cache_dir / 'direct' / 'final-model.pt')
+                cached_path(
+                    "http://www.redewiedergabe.de/models/direct.zip",
+                    cache_dir=cache_dir,
+                )
+                unzip_file(
+                    Path(flair.cache_root) / cache_dir / "direct.zip",
+                    Path(flair.cache_root) / cache_dir,
+                )
+            model_name = str(Path(flair.cache_root) / cache_dir / "direct" / "final-model.pt")
 
         if model_name == "de-historic-reported":
-            model_file = Path(flair.cache_root)  / cache_dir / 'reported' / 'final-model.pt'
+            model_file = Path(flair.cache_root) / cache_dir / "reported" / "final-model.pt"
             if not model_file.exists():
-                cached_path('http://www.redewiedergabe.de/models/reported.zip', cache_dir=cache_dir)
-                unzip_file(Path(flair.cache_root)  / cache_dir / 'reported.zip', Path(flair.cache_root)  / cache_dir)
-            model_name = str(Path(flair.cache_root)  / cache_dir / 'reported' / 'final-model.pt')
+                cached_path(
+                    "http://www.redewiedergabe.de/models/reported.zip",
+                    cache_dir=cache_dir,
+                )
+                unzip_file(
+                    Path(flair.cache_root) / cache_dir / "reported.zip",
+                    Path(flair.cache_root) / cache_dir,
+                )
+            model_name = str(Path(flair.cache_root) / cache_dir / "reported" / "final-model.pt")
 
         if model_name == "de-historic-free-indirect":
-            model_file = Path(flair.cache_root)  / cache_dir / 'freeIndirect' / 'final-model.pt'
+            model_file = Path(flair.cache_root) / cache_dir / "freeIndirect" / "final-model.pt"
             if not model_file.exists():
-                cached_path('http://www.redewiedergabe.de/models/freeIndirect.zip', cache_dir=cache_dir)
-                unzip_file(Path(flair.cache_root)  / cache_dir / 'freeIndirect.zip', Path(flair.cache_root)  / cache_dir)
-            model_name = str(Path(flair.cache_root)  / cache_dir / 'freeIndirect' / 'final-model.pt')
+                cached_path(
+                    "http://www.redewiedergabe.de/models/freeIndirect.zip",
+                    cache_dir=cache_dir,
+                )
+                unzip_file(
+                    Path(flair.cache_root) / cache_dir / "freeIndirect.zip",
+                    Path(flair.cache_root) / cache_dir,
+                )
+            model_name = str(Path(flair.cache_root) / cache_dir / "freeIndirect" / "final-model.pt")
 
         return model_name
 
@@ -1153,7 +1101,9 @@ class MemoryTagger(flair.nn.Model):
         print(tabulate(data, headers=["FROM", "TO", "SCORE"]))
 
     def __str__(self):
-        return super(flair.nn.Model, self).__str__().rstrip(')') + \
-               f'  (beta): {self.beta}\n' + \
-               f'  (weights): {self.weight_dict}\n' + \
-               f'  (weight_tensor) {self.loss_weights}\n)'
+        return (
+            super(flair.nn.Model, self).__str__().rstrip(")")
+            + f"  (beta): {self.beta}\n"
+            + f"  (weights): {self.weight_dict}\n"
+            + f"  (weight_tensor) {self.loss_weights}\n)"
+        )
