@@ -1,26 +1,25 @@
+import math
+import random
 from random import randint
 
 import torch
-import torch.nn.functional as F
 import torch.nn as nn
-import math
-
-from torch.nn import GRUCell, GRU, LSTMCell
+import torch.nn.functional as F
+from torch.nn import GRU, GRUCell, LSTMCell
 
 import flair
-import random
 
 
 class MemoryLSTM(nn.Module):
     def __init__(
-            self,
-            input_size,
-            hidden_size,
-            memory_size,
-            word_drop_probability: float = 0.2,
-            bias: bool = True,
-            batch_first: bool = False,
-            bidirectional: bool = False,
+        self,
+        input_size,
+        hidden_size,
+        memory_size,
+        word_drop_probability: float = 0.2,
+        bias: bool = True,
+        batch_first: bool = False,
+        bidirectional: bool = False,
     ):
         super().__init__()
 
@@ -39,7 +38,9 @@ class MemoryLSTM(nn.Module):
 
         # initialize cell of forward LSTM
         self.forward_cell = LSTMCell(
-            input_size, hidden_size, bias,
+            input_size,
+            hidden_size,
+            bias,
         )
 
         # map to record the last state of each word
@@ -58,7 +59,9 @@ class MemoryLSTM(nn.Module):
             backwards_input_size = input_size if independent_rnns else hidden_size
 
             self.backward_cell = LSTMCell(
-                backwards_input_size, hidden_size, bias,
+                backwards_input_size,
+                hidden_size,
+                bias,
             )
 
             # map to record the last state of each word in backward pass
@@ -77,9 +80,7 @@ class MemoryLSTM(nn.Module):
     def train(self, mode=True):
         super().train(mode=mode)
 
-        print(
-            f"Memsize: {len(self.word_memory_forward)}"
-        )
+        print(f"Memsize: {len(self.word_memory_forward)}")
         # print(
         #     f"Memsize: {len(self.word_memory_count)}"
         # )
@@ -96,10 +97,8 @@ class MemoryLSTM(nn.Module):
             #
             # for key in delete_keys:
             #     del self.word_memory_forward[key]
-                    # print(key)
-            print(
-                f"Train mode - resetting memory. Memsize: {len(self.word_memory_forward)}"
-            )
+            # print(key)
+            print(f"Train mode - resetting memory. Memsize: {len(self.word_memory_forward)}")
             self.word_memory_count = {}
         else:
             self.forward_store = self.word_memory_forward.copy()
@@ -119,12 +118,20 @@ class MemoryLSTM(nn.Module):
         #     # memory is wiped each time we do evaluation
         #     print(f"Prediction mode - growing memory")
 
-    def memory_rnn_pass(self, rnn_cell, memory, input_, batch_dp_names, hidden=None, is_forward: bool = True):
+    def memory_rnn_pass(
+        self,
+        rnn_cell,
+        memory,
+        input_,
+        batch_dp_names,
+        hidden=None,
+        is_forward: bool = True,
+    ):
         outputs = []
 
         if hidden is not None:
             if input_.size(0) < hidden.size(0):
-                hidden = hidden[:input_.size(0)]
+                hidden = hidden[: input_.size(0)]
 
         word_dim = 1 if self.batch_first else 0
         sentence_dim = 0 if self.batch_first else 1
@@ -145,8 +152,9 @@ class MemoryLSTM(nn.Module):
                 word_indices = reversed(word_indices)
 
             # get the previous hidden and cell state (initialize to zero if none)
-            previous_hidden_state = self.initial_hidden if hidden is None else hidden[sentence].unsqueeze(0)[
-                0]  # input, (h_0, c_0)
+            previous_hidden_state = (
+                self.initial_hidden if hidden is None else hidden[sentence].unsqueeze(0)[0]
+            )  # input, (h_0, c_0)
             previous_cell_state = self.initial_cell if hidden is None else hidden[sentence].unsqueeze(0)[1]
 
             # print(row)
@@ -182,8 +190,8 @@ class MemoryLSTM(nn.Module):
                     # cell state is combination of previous cell state, and last cell state of this word
                     previous_cell_state = torch.cat(
                         [
-                            previous_cell_state[0, 0: previous_cell_size],
-                            word_memory_cell_state
+                            previous_cell_state[0, 0:previous_cell_size],
+                            word_memory_cell_state,
                         ]
                     ).unsqueeze(0)
 
@@ -198,7 +206,10 @@ class MemoryLSTM(nn.Module):
                     #     print(m.forward(previous_cell_state))
                     #     print()
 
-                    input_states = (previous_hidden_state, previous_cell_state)  # input, (h_0, c_0)
+                    input_states = (
+                        previous_hidden_state,
+                        previous_cell_state,
+                    )  # input, (h_0, c_0)
 
                     # send through RNN
                     hidden_state_out = rnn_cell(data_point, input_states)
@@ -207,7 +218,7 @@ class MemoryLSTM(nn.Module):
                     previous_hidden_state = hidden_state_out[0]
 
                     # split off memory portion and store
-                    memory_hidden = previous_cell_state[0, previous_cell_size:previous_cell_size + self.memory_size]
+                    memory_hidden = previous_cell_state[0, previous_cell_size : previous_cell_size + self.memory_size]
                     memory[token_text] = memory_hidden
 
                     column_outputs.append(previous_hidden_state)
@@ -241,22 +252,38 @@ class MemoryLSTM(nn.Module):
             for key in self.word_memory_backward.keys():
                 self.word_memory_backward[key] = self.word_memory_backward[key].detach()
 
-            backward, all_last_hidden = self.memory_rnn_pass(self.backward_cell, self.word_memory_backward, input_,
-                                                             batch_dp_names, hidden, is_forward=False)
+            backward, all_last_hidden = self.memory_rnn_pass(
+                self.backward_cell,
+                self.word_memory_backward,
+                input_,
+                batch_dp_names,
+                hidden,
+                is_forward=False,
+            )
 
             # detach all hidden states in memory
             # for key in self.word_memory_forward.keys():
             #     self.word_memory_forward[key] = self.word_memory_forward[key].detach()
 
-            forward, all_last_hidden = self.memory_rnn_pass(self.forward_cell, self.word_memory_forward, input_,
-                                                            batch_dp_names, hidden, is_forward=True)
+            forward, all_last_hidden = self.memory_rnn_pass(
+                self.forward_cell,
+                self.word_memory_forward,
+                input_,
+                batch_dp_names,
+                hidden,
+                is_forward=True,
+            )
 
             hidden_states_for_all_positions = torch.cat([forward, backward], dim=2)
         else:
-            hidden_states_for_all_positions, all_last_hidden = self.memory_rnn_pass(self.forward_cell,
-                                                                                    self.word_memory_forward, input_,
-                                                                                    batch_dp_names, hidden,
-                                                                                    is_forward=True)
+            hidden_states_for_all_positions, all_last_hidden = self.memory_rnn_pass(
+                self.forward_cell,
+                self.word_memory_forward,
+                input_,
+                batch_dp_names,
+                hidden,
+                is_forward=True,
+            )
 
         delete_keys = {}
         if self.training:
@@ -264,10 +291,10 @@ class MemoryLSTM(nn.Module):
                 for text in batch:
                     # word_count = 1 if text not in self.word_memory_count else self.word_memory_count[text] + 1
                     # self.word_memory_count[text] = word_count
-                    delete_keys[text] = 1.
+                    delete_keys[text] = 1.0
 
         # detach all hidden states in memory
         for key in delete_keys.keys():
             self.word_memory_forward[key] = self.word_memory_forward[key].detach()
 
-        return hidden_states_for_all_positions, all_last_hidden
\ No newline at end of file
+        return hidden_states_for_all_positions, all_last_hidden
