@@ -1,9 +1,6 @@
 import logging
 from pathlib import Path
-
-import logging
-from pathlib import Path
-from typing import List, Union, Optional, Callable
+from typing import Callable, List, Optional, Union
 
 import numpy as np
 import torch
@@ -14,7 +11,7 @@ from torch.utils.data import DataLoader
 from tqdm import tqdm
 
 import flair.nn
-from flair.data import Dictionary, Sentence, Token, Label, space_tokenizer
+from flair.data import Dictionary, Label, Sentence, Token, space_tokenizer
 from flair.datasets import SentenceDataset, StringDataset
 from flair.embeddings import TokenEmbeddings
 from flair.file_utils import cached_path
@@ -154,9 +151,7 @@ class SequenceTagger(flair.nn.Model):
             num_directions = 2 if self.bidirectional else 1
 
             # final linear map to tag space
-            self.linear = torch.nn.Linear(
-                hidden_size * num_directions, len(tag_dictionary)
-            )
+            self.linear = torch.nn.Linear(hidden_size * num_directions, len(tag_dictionary))
 
             if self.rnn_type in ["LSTM", "GRU"]:
 
@@ -173,17 +168,11 @@ class SequenceTagger(flair.nn.Model):
             self.linear = torch.nn.Linear(self.rnn_input_dim, len(tag_dictionary))
 
         if self.use_crf:
-            self.transitions = torch.nn.Parameter(
-                torch.randn(self.tagset_size, self.tagset_size)
-            )
+            self.transitions = torch.nn.Parameter(torch.randn(self.tagset_size, self.tagset_size))
 
-            self.transitions.detach()[
-                self.tag_dictionary.get_idx_for_item(START_TAG), :
-            ] = -10000
+            self.transitions.detach()[self.tag_dictionary.get_idx_for_item(START_TAG), :] = -10000
 
-            self.transitions.detach()[
-                :, self.tag_dictionary.get_idx_for_item(STOP_TAG)
-            ] = -10000
+            self.transitions.detach()[:, self.tag_dictionary.get_idx_for_item(STOP_TAG)] = -10000
 
         self.to(flair.device)
 
@@ -212,18 +201,10 @@ class SequenceTagger(flair.nn.Model):
 
         rnn_type = "LSTM" if not "rnn_type" in state.keys() else state["rnn_type"]
         use_dropout = 0.0 if not "use_dropout" in state.keys() else state["use_dropout"]
-        use_word_dropout = (
-            0.0 if not "use_word_dropout" in state.keys() else state["use_word_dropout"]
-        )
-        use_locked_dropout = (
-            0.0
-            if not "use_locked_dropout" in state.keys()
-            else state["use_locked_dropout"]
-        )
+        use_word_dropout = 0.0 if not "use_word_dropout" in state.keys() else state["use_word_dropout"]
+        use_locked_dropout = 0.0 if not "use_locked_dropout" in state.keys() else state["use_locked_dropout"]
         train_initial_hidden_state = (
-            False
-            if not "train_initial_hidden_state" in state.keys()
-            else state["train_initial_hidden_state"]
+            False if not "train_initial_hidden_state" in state.keys() else state["train_initial_hidden_state"]
         )
 
         model = SequenceTagger(
@@ -287,28 +268,18 @@ class SequenceTagger(flair.nn.Model):
                 )
 
             # reverse sort all sequences by their length
-            rev_order_len_index = sorted(
-                range(len(sentences)), key=lambda k: len(sentences[k]), reverse=True
-            )
-            original_order_index = sorted(
-                range(len(rev_order_len_index)), key=lambda k: rev_order_len_index[k]
-            )
+            rev_order_len_index = sorted(range(len(sentences)), key=lambda k: len(sentences[k]), reverse=True)
+            original_order_index = sorted(range(len(rev_order_len_index)), key=lambda k: rev_order_len_index[k])
 
-            reordered_sentences: List[Union[Sentence, str]] = [
-                sentences[index] for index in rev_order_len_index
-            ]
+            reordered_sentences: List[Union[Sentence, str]] = [sentences[index] for index in rev_order_len_index]
 
             if isinstance(sentences[0], Sentence):
                 # remove previous embeddings
                 store_embeddings(reordered_sentences, "none")
                 dataset = SentenceDataset(reordered_sentences)
             else:
-                dataset = StringDataset(
-                    reordered_sentences, use_tokenizer=use_tokenizer
-                )
-            dataloader = DataLoader(
-                dataset=dataset, batch_size=mini_batch_size, collate_fn=lambda x: x
-            )
+                dataset = StringDataset(reordered_sentences, use_tokenizer=use_tokenizer)
+            dataloader = DataLoader(dataset=dataset, batch_size=mini_batch_size, collate_fn=lambda x: x)
 
             if self.use_crf:
                 transitions = self.transitions.detach().cpu().numpy()
@@ -350,9 +321,7 @@ class SequenceTagger(flair.nn.Model):
                 # clearing token embeddings to save memory
                 store_embeddings(batch, storage_mode=embedding_storage_mode)
 
-            results: List[Union[Sentence, str]] = [
-                results[index] for index in original_order_index
-            ]
+            results: List[Union[Sentence, str]] = [results[index] for index in original_order_index]
             assert len(sentences) == len(results)
             return results
 
@@ -411,13 +380,9 @@ class SequenceTagger(flair.nn.Model):
                     lines.append("\n")
                 for sentence in batch:
                     # make list of gold tags
-                    gold_tags = [
-                        (tag.tag, str(tag)) for tag in sentence.get_spans(self.tag_type)
-                    ]
+                    gold_tags = [(tag.tag, str(tag)) for tag in sentence.get_spans(self.tag_type)]
                     # make list of predicted tags
-                    predicted_tags = [
-                        (tag.tag, str(tag)) for tag in sentence.get_spans("predicted")
-                    ]
+                    predicted_tags = [(tag.tag, str(tag)) for tag in sentence.get_spans("predicted")]
 
                     # check for true positives, false positives and false negatives
                     for tag, prediction in predicted_tags:
@@ -462,9 +427,7 @@ class SequenceTagger(flair.nn.Model):
 
             return result, eval_loss
 
-    def forward_loss(
-        self, data_points: Union[List[Sentence], Sentence], sort=True
-    ) -> torch.tensor:
+    def forward_loss(self, data_points: Union[List[Sentence], Sentence], sort=True) -> torch.tensor:
         features = self.forward(data_points)
         return self._calculate_loss(features, data_points)
 
@@ -495,7 +458,7 @@ class SequenceTagger(flair.nn.Model):
                     word_count = self.word_memory_count[token.text]
 
                     word_class_softmax = self.word_memory_global[token.text] / word_count
-                    word_frequency_quotient = 1. if word_count >= 10. else word_count / 10.
+                    word_frequency_quotient = 1.0 if word_count >= 10.0 else word_count / 10.0
 
                     # vector = torch.cat([word_class_softmax, torch.tensor(word_frequency_quotient)])
 
@@ -515,9 +478,7 @@ class SequenceTagger(flair.nn.Model):
         all_embs = list()
         for sentence in sentences:
             t_ids.append([token.text for token in sentence])
-            all_embs += [
-                emb for token in sentence for emb in token.get_each_embedding()
-            ]
+            all_embs += [emb for token in sentence for emb in token.get_each_embedding()]
             nb_padding_tokens = longest_token_sequence_in_batch - len(sentence)
 
             if nb_padding_tokens > 0:
@@ -525,7 +486,11 @@ class SequenceTagger(flair.nn.Model):
                 all_embs.append(t)
 
         sentence_tensor = torch.cat(all_embs).view(
-            [len(sentences), longest_token_sequence_in_batch, self.embeddings.embedding_length]
+            [
+                len(sentences),
+                longest_token_sequence_in_batch,
+                self.embeddings.embedding_length,
+            ]
         )
 
         # --------------------------------------------------------------------
@@ -580,9 +545,11 @@ class SequenceTagger(flair.nn.Model):
                 word_label_dist = softmax_batch[s_id, t_id].detach().clone()
                 self.word_memory_last[token.text] = word_label_dist
 
-                word_prior = self.word_memory_global[token.text] + word_label_dist \
-                    if token.text in self.word_memory_global \
+                word_prior = (
+                    self.word_memory_global[token.text] + word_label_dist
+                    if token.text in self.word_memory_global
                     else torch.zeros(len(self.tag_dictionary))
+                )
                 word_count = 1 if token.text not in self.word_memory_count else self.word_memory_count[token.text] + 1
 
                 self.word_memory_global[token.text] = word_prior
@@ -594,28 +561,21 @@ class SequenceTagger(flair.nn.Model):
                 #     if softmax_batch[s_id, t_id][tag] > 0.1:
                 #         print(f' - {self.tag_dictionary.get_item_for_index(tag)}: {softmax_batch[s_id, t_id][tag]}')
 
-
         return features
 
     def _score_sentence(self, feats, tags, lens_):
 
-        start = torch.tensor(
-            [self.tag_dictionary.get_idx_for_item(START_TAG)], device=flair.device
-        )
+        start = torch.tensor([self.tag_dictionary.get_idx_for_item(START_TAG)], device=flair.device)
         start = start[None, :].repeat(tags.shape[0], 1)
 
-        stop = torch.tensor(
-            [self.tag_dictionary.get_idx_for_item(STOP_TAG)], device=flair.device
-        )
+        stop = torch.tensor([self.tag_dictionary.get_idx_for_item(STOP_TAG)], device=flair.device)
         stop = stop[None, :].repeat(tags.shape[0], 1)
 
         pad_start_tags = torch.cat([start, tags], 1)
         pad_stop_tags = torch.cat([tags, stop], 1)
 
         for i in range(len(lens_)):
-            pad_stop_tags[i, lens_[i] :] = self.tag_dictionary.get_idx_for_item(
-                STOP_TAG
-            )
+            pad_stop_tags[i, lens_[i] :] = self.tag_dictionary.get_idx_for_item(STOP_TAG)
 
         score = torch.FloatTensor(feats.shape[0]).to(flair.device)
 
@@ -623,16 +583,12 @@ class SequenceTagger(flair.nn.Model):
             r = torch.LongTensor(range(lens_[i])).to(flair.device)
 
             score[i] = torch.sum(
-                self.transitions[
-                    pad_stop_tags[i, : lens_[i] + 1], pad_start_tags[i, : lens_[i] + 1]
-                ]
+                self.transitions[pad_stop_tags[i, : lens_[i] + 1], pad_start_tags[i, : lens_[i] + 1]]
             ) + torch.sum(feats[i, r, tags[i, : lens_[i]]])
 
         return score
 
-    def _calculate_loss(
-        self, features: torch.tensor, sentences: List[Sentence]
-    ) -> float:
+    def _calculate_loss(self, features: torch.tensor, sentences: List[Sentence]) -> float:
 
         lengths: List[int] = [len(sentence.tokens) for sentence in sentences]
 
@@ -640,8 +596,7 @@ class SequenceTagger(flair.nn.Model):
         for s_id, sentence in enumerate(sentences):
             # get the tags in this sentence
             tag_idx: List[int] = [
-                self.tag_dictionary.get_idx_for_item(token.get_tag(self.tag_type).value)
-                for token in sentence
+                self.tag_dictionary.get_idx_for_item(token.get_tag(self.tag_type).value) for token in sentence
             ]
             # add tags as tensor
             tag = torch.tensor(tag_idx, device=flair.device)
@@ -660,14 +615,10 @@ class SequenceTagger(flair.nn.Model):
 
         else:
             score = 0
-            for sentence_feats, sentence_tags, sentence_length in zip(
-                features, tag_list, lengths
-            ):
+            for sentence_feats, sentence_tags, sentence_length in zip(features, tag_list, lengths):
                 sentence_feats = sentence_feats[:sentence_length]
 
-                score += torch.nn.functional.cross_entropy(
-                    sentence_feats, sentence_tags
-                )
+                score += torch.nn.functional.cross_entropy(sentence_feats, sentence_tags)
             score /= len(features)
             return score
 
@@ -713,19 +664,14 @@ class SequenceTagger(flair.nn.Model):
                 scores = softmax[:length].tolist()
 
             tags.append(
-                [
-                    Label(self.tag_dictionary.get_item_for_index(tag), conf)
-                    for conf, tag in zip(confidences, tag_seq)
-                ]
+                [Label(self.tag_dictionary.get_item_for_index(tag), conf) for conf, tag in zip(confidences, tag_seq)]
             )
 
             if get_all_tags:
                 all_tags.append(
                     [
                         [
-                            Label(
-                                self.tag_dictionary.get_item_for_index(score_id), score
-                            )
+                            Label(self.tag_dictionary.get_item_for_index(score_id), score)
                             for score_id, score in enumerate(score_dist)
                         ]
                         for score_dist in scores
@@ -741,20 +687,14 @@ class SequenceTagger(flair.nn.Model):
         y = np.exp(x_norm)
         return y / y.sum(axis=axis, keepdims=True)
 
-    def _viterbi_decode(
-        self, feats: np.ndarray, transitions: np.ndarray, all_scores: bool
-    ):
+    def _viterbi_decode(self, feats: np.ndarray, transitions: np.ndarray, all_scores: bool):
         id_start = self.tag_dictionary.get_idx_for_item(START_TAG)
         id_stop = self.tag_dictionary.get_idx_for_item(STOP_TAG)
 
         backpointers = np.empty(shape=(feats.shape[0], self.tagset_size), dtype=np.int_)
-        backscores = np.empty(
-            shape=(feats.shape[0], self.tagset_size), dtype=np.float32
-        )
+        backscores = np.empty(shape=(feats.shape[0], self.tagset_size), dtype=np.float32)
 
-        init_vvars = np.expand_dims(
-            np.repeat(-10000.0, self.tagset_size), axis=0
-        ).astype(np.float32)
+        init_vvars = np.expand_dims(np.repeat(-10000.0, self.tagset_size), axis=0).astype(np.float32)
         init_vvars[0][id_start] = 0
 
         forward_var = init_vvars
@@ -792,17 +732,13 @@ class SequenceTagger(flair.nn.Model):
             for index, (tag_id, tag_scores) in enumerate(zip(best_path, all_scores_np)):
                 if type(tag_id) != int and tag_id.item() != tag_scores.argmax():
                     swap_index_score = tag_scores.argmax()
-                    all_scores_np[index][tag_id.item()], all_scores_np[index][
-                        swap_index_score
-                    ] = (
+                    (all_scores_np[index][tag_id.item()], all_scores_np[index][swap_index_score],) = (
                         all_scores_np[index][swap_index_score],
                         all_scores_np[index][tag_id.item()],
                     )
                 elif type(tag_id) == int and tag_id != tag_scores.argmax():
                     swap_index_score = tag_scores.argmax()
-                    all_scores_np[index][tag_id], all_scores_np[index][
-                        swap_index_score
-                    ] = (
+                    (all_scores_np[index][tag_id], all_scores_np[index][swap_index_score],) = (
                         all_scores_np[index][swap_index_score],
                         all_scores_np[index][tag_id],
                     )
@@ -824,9 +760,9 @@ class SequenceTagger(flair.nn.Model):
 
         forward_var[:, 0, :] = init_alphas[None, :].repeat(feats.shape[0], 1)
 
-        transitions = self.transitions.view(
-            1, self.transitions.shape[0], self.transitions.shape[1]
-        ).repeat(feats.shape[0], 1, 1)
+        transitions = self.transitions.view(1, self.transitions.shape[0], self.transitions.shape[1]).repeat(
+            feats.shape[0], 1, 1
+        )
 
         for i in range(feats.shape[1]):
             emit_score = feats[:, i, :]
@@ -834,16 +770,12 @@ class SequenceTagger(flair.nn.Model):
             tag_var = (
                 emit_score[:, :, None].repeat(1, 1, transitions.shape[2])
                 + transitions
-                + forward_var[:, i, :][:, :, None]
-                .repeat(1, 1, transitions.shape[2])
-                .transpose(2, 1)
+                + forward_var[:, i, :][:, :, None].repeat(1, 1, transitions.shape[2]).transpose(2, 1)
             )
 
             max_tag_var, _ = torch.max(tag_var, dim=2)
 
-            tag_var = tag_var - max_tag_var[:, :, None].repeat(
-                1, 1, transitions.shape[2]
-            )
+            tag_var = tag_var - max_tag_var[:, :, None].repeat(1, 1, transitions.shape[2])
 
             agg_ = torch.log(torch.sum(torch.exp(tag_var), dim=2))
 
@@ -854,9 +786,9 @@ class SequenceTagger(flair.nn.Model):
 
         forward_var = forward_var[range(forward_var.shape[0]), lens_, :]
 
-        terminal_var = forward_var + self.transitions[
-            self.tag_dictionary.get_idx_for_item(STOP_TAG)
-        ][None, :].repeat(forward_var.shape[0], 1)
+        terminal_var = forward_var + self.transitions[self.tag_dictionary.get_idx_for_item(STOP_TAG)][None, :].repeat(
+            forward_var.shape[0], 1
+        )
 
         alpha = log_sum_exp_batch(terminal_var)
 
@@ -866,33 +798,23 @@ class SequenceTagger(flair.nn.Model):
     def _filter_empty_sentences(sentences: List[Sentence]) -> List[Sentence]:
         filtered_sentences = [sentence for sentence in sentences if sentence.tokens]
         if len(sentences) != len(filtered_sentences):
-            log.warning(
-                "Ignore {} sentence(s) with no tokens.".format(
-                    len(sentences) - len(filtered_sentences)
-                )
-            )
+            log.warning("Ignore {} sentence(s) with no tokens.".format(len(sentences) - len(filtered_sentences)))
         return filtered_sentences
 
     @staticmethod
     def _filter_empty_string(texts: List[str]) -> List[str]:
         filtered_texts = [text for text in texts if text]
         if len(texts) != len(filtered_texts):
-            log.warning(
-                f"Ignore {len(texts) - len(filtered_texts)} string(s) with no tokens."
-            )
+            log.warning(f"Ignore {len(texts) - len(filtered_texts)} string(s) with no tokens.")
         return filtered_texts
 
     def _fetch_model(model_name) -> str:
 
         model_map = {}
 
-        aws_resource_path_v04 = (
-            "https://s3.eu-central-1.amazonaws.com/alan-nlp/resources/models-v0.4"
-        )
+        aws_resource_path_v04 = "https://s3.eu-central-1.amazonaws.com/alan-nlp/resources/models-v0.4"
 
-        model_map["ner"] = "/".join(
-            [aws_resource_path_v04, "NER-conll03-english", "en-ner-conll03-v0.4.pt"]
-        )
+        model_map["ner"] = "/".join([aws_resource_path_v04, "NER-conll03-english", "en-ner-conll03-v0.4.pt"])
 
         model_map["ner-fast"] = "/".join(
             [
@@ -928,9 +850,7 @@ class SequenceTagger(flair.nn.Model):
             )
 
         for key in ["ner-multi-fast", "multi-ner-fast"]:
-            model_map[key] = "/".join(
-                [aws_resource_path_v04, "NER-multi-fast", "ner-multi-fast.pt"]
-            )
+            model_map[key] = "/".join([aws_resource_path_v04, "NER-multi-fast", "ner-multi-fast.pt"])
 
         for key in ["ner-multi-fast-learn", "multi-ner-fast-learn"]:
             model_map[key] = "/".join(
@@ -967,13 +887,9 @@ class SequenceTagger(flair.nn.Model):
             )
 
         for key in ["pos-multi-fast", "multi-pos-fast"]:
-            model_map[key] = "/".join(
-                [aws_resource_path_v04, "UPOS-multi-fast", "pos-multi-fast.pt"]
-            )
+            model_map[key] = "/".join([aws_resource_path_v04, "UPOS-multi-fast", "pos-multi-fast.pt"])
 
-        model_map["frame"] = "/".join(
-            [aws_resource_path_v04, "release-frame-1", "en-frame-ontonotes-v0.4.pt"]
-        )
+        model_map["frame"] = "/".join([aws_resource_path_v04, "release-frame-1", "en-frame-ontonotes-v0.4.pt"])
 
         model_map["frame-fast"] = "/".join(
             [
@@ -999,17 +915,11 @@ class SequenceTagger(flair.nn.Model):
             ]
         )
 
-        model_map["da-pos"] = "/".join(
-            [aws_resource_path_v04, "POS-danish", "da-pos-v0.1.pt"]
-        )
+        model_map["da-pos"] = "/".join([aws_resource_path_v04, "POS-danish", "da-pos-v0.1.pt"])
 
-        model_map["da-ner"] = "/".join(
-            [aws_resource_path_v04, "NER-danish", "da-ner-v0.1.pt"]
-        )
+        model_map["da-ner"] = "/".join([aws_resource_path_v04, "NER-danish", "da-ner-v0.1.pt"])
 
-        model_map["de-pos"] = "/".join(
-            [aws_resource_path_v04, "release-de-pos-0", "de-pos-ud-hdt-v0.4.pt"]
-        )
+        model_map["de-pos"] = "/".join([aws_resource_path_v04, "release-de-pos-0", "de-pos-ud-hdt-v0.4.pt"])
 
         model_map["de-pos-fine-grained"] = "/".join(
             [
@@ -1019,20 +929,12 @@ class SequenceTagger(flair.nn.Model):
             ]
         )
 
-        model_map["de-ner"] = "/".join(
-            [aws_resource_path_v04, "release-de-ner-0", "de-ner-conll03-v0.4.pt"]
-        )
+        model_map["de-ner"] = "/".join([aws_resource_path_v04, "release-de-ner-0", "de-ner-conll03-v0.4.pt"])
 
-        model_map["de-ner-germeval"] = "/".join(
-            [aws_resource_path_v04, "NER-germeval", "de-ner-germeval-0.4.1.pt"]
-        )
+        model_map["de-ner-germeval"] = "/".join([aws_resource_path_v04, "NER-germeval", "de-ner-germeval-0.4.1.pt"])
 
-        model_map["fr-ner"] = "/".join(
-            [aws_resource_path_v04, "release-fr-ner-0", "fr-ner-wikiner-0.4.pt"]
-        )
-        model_map["nl-ner"] = "/".join(
-            [aws_resource_path_v04, "NER-conll2002-dutch", "nl-ner-conll02-v0.1.pt"]
-        )
+        model_map["fr-ner"] = "/".join([aws_resource_path_v04, "release-fr-ner-0", "fr-ner-wikiner-0.4.pt"])
+        model_map["nl-ner"] = "/".join([aws_resource_path_v04, "NER-conll2002-dutch", "nl-ner-conll02-v0.1.pt"])
 
         cache_dir = Path("models")
         if model_name in model_map:
@@ -1060,9 +962,7 @@ class SequenceTagger(flair.nn.Model):
             self.word_memory_last = {}
             self.word_memory_global = {}
             self.word_memory_count = {}
-            print(
-                f"Train mode - resetting memory."
-            )
+            print(f"Train mode - resetting memory.")
         else:
             # memory is wiped each time we do evaluation
-            print(f"Prediction mode - growing memory")
\ No newline at end of file
+            print(f"Prediction mode - growing memory")
