@@ -1,6 +1,6 @@
 import logging
 from pathlib import Path
-from typing import List, Union, Optional, Callable, Dict
+from typing import Callable, Dict, List, Optional, Union
 
 import numpy as np
 import torch
@@ -10,16 +10,16 @@ from torch.utils.data.dataset import Dataset
 from tqdm import tqdm
 
 import flair.nn
-from flair.data import Dictionary, Sentence, Token, Label, space_tokenizer, DataPoint
-from flair.datasets import SentenceDataset, DataLoader
+from flair.data import DataPoint, Dictionary, Label, Sentence, Token, space_tokenizer
+from flair.datasets import DataLoader, SentenceDataset
 from flair.embeddings import TokenEmbeddings
 from flair.models import SequenceTagger
 from flair.training_utils import Metric, Result, store_embeddings
 
 log = logging.getLogger("flair")
 
-class BiaffineAttention(torch.nn.Module):
 
+class BiaffineAttention(torch.nn.Module):
     def __init__(self, input_dim, output_dim):
         super(BiaffineAttention, self).__init__()
         self.input_dim = input_dim
@@ -48,7 +48,7 @@ class BiaffineTagger(flair.nn.Model):
         dropout: float = 0.0,
         word_dropout: float = 0.05,
         locked_dropout: float = 0.5,
-        reproject_embeddings: Union[bool,int] = True,
+        reproject_embeddings: Union[bool, int] = True,
         rnn_type: str = "LSTM",
     ):
         """
@@ -123,7 +123,7 @@ class BiaffineTagger(flair.nn.Model):
                 bidirectional=True,
                 batch_first=True,
             )
-        rnn_output_size = 2 * hidden_size # RNN is bidirectional
+        rnn_output_size = 2 * hidden_size  # RNN is bidirectional
 
         # maps to start and stop features
         self.ffnn_start = torch.nn.Linear(rnn_output_size, biaffine_size)
@@ -139,7 +139,7 @@ class BiaffineTagger(flair.nn.Model):
     #     features = self.forward(data_points)
     #     return self._calculate_loss(features, data_points)
 
-    def forward_loss(self, sentences: List[Sentence], also_return_logits = False):
+    def forward_loss(self, sentences: List[Sentence], also_return_logits=False):
 
         # embed sentences
         self.embeddings.embed(sentences)
@@ -169,9 +169,7 @@ class BiaffineTagger(flair.nn.Model):
 
         # print(rnn_output.data.size())
         # asd
-        sentence_tensor, output_lengths = torch.nn.utils.rnn.pad_packed_sequence(
-            rnn_output, batch_first=True
-        )
+        sentence_tensor, output_lengths = torch.nn.utils.rnn.pad_packed_sequence(rnn_output, batch_first=True)
 
         # more dropout after RNN
         if self.use_dropout > 0.0:
@@ -204,16 +202,20 @@ class BiaffineTagger(flair.nn.Model):
         # print(softmax_batch)
         # asd
         # predicted = torch.zeros(result_matrix.size(), dtype=torch.long, device=flair.device)
-        gold = torch.zeros([result_matrix.size(0), result_matrix.size(1), result_matrix.size(2)], dtype=torch.long, device=flair.device)
+        gold = torch.zeros(
+            [result_matrix.size(0), result_matrix.size(1), result_matrix.size(2)],
+            dtype=torch.long,
+            device=flair.device,
+        )
         for s_id, sentence in enumerate(sentences):
-            for span in sentence.get_spans('ner'):
-                gold[s_id, span[0].idx-1, span[-1].idx-1] = self.tag_dictionary.get_idx_for_item(span.tag)
+            for span in sentence.get_spans("ner"):
+                gold[s_id, span[0].idx - 1, span[-1].idx - 1] = self.tag_dictionary.get_idx_for_item(span.tag)
             # print(f'length sentence: {len(sentence)}')
             for i in range(len(sentence), max(lengths)):
                 # print(i)
                 for y in range(len(sentence), max(lengths)):
                     # print(y)
-                    result_matrix[s_id, i, y] = 0.
+                    result_matrix[s_id, i, y] = 0.0
                     # print(result_matrix[s_id, i, y])
 
         print(result_matrix.size())
@@ -237,16 +239,15 @@ class BiaffineTagger(flair.nn.Model):
 
         loss = torch.nn.functional.cross_entropy(
             result_matrix.contiguous().view(len(sentences) * matrix_size, 5),
-            gold.contiguous().view(len(sentences) * matrix_size))
+            gold.contiguous().view(len(sentences) * matrix_size),
+        )
         #
         if also_return_logits:
             return result_matrix, loss
         #
         return loss
 
-    def _calculate_loss(
-        self, features: torch.tensor, sentences: List[Sentence]
-    ) -> float:
+    def _calculate_loss(self, features: torch.tensor, sentences: List[Sentence]) -> float:
 
         lengths: List[int] = [len(sentence.tokens) for sentence in sentences]
 
@@ -254,21 +255,16 @@ class BiaffineTagger(flair.nn.Model):
         for s_id, sentence in enumerate(sentences):
             # get the tags in this sentence
             tag_idx: List[int] = [
-                self.tag_dictionary.get_idx_for_item(token.get_tag(self.tag_type).value)
-                for token in sentence
+                self.tag_dictionary.get_idx_for_item(token.get_tag(self.tag_type).value) for token in sentence
             ]
             # add tags as tensor
             tag = torch.tensor(tag_idx, device=flair.device)
             tag_list.append(tag)
 
         score = 0
-        for sentence_feats, sentence_tags, sentence_length in zip(
-            features, tag_list, lengths
-        ):
+        for sentence_feats, sentence_tags, sentence_length in zip(features, tag_list, lengths):
             sentence_feats = sentence_feats[:sentence_length]
-            score += torch.nn.functional.cross_entropy(
-                sentence_feats, sentence_tags
-            )
+            score += torch.nn.functional.cross_entropy(sentence_feats, sentence_tags)
         score /= len(features)
         return score
 
@@ -293,13 +289,14 @@ class BiaffineTagger(flair.nn.Model):
 
         for s_no, sentence in enumerate(batch_sentences):
             for token in sentence:
-                token.set_label(label_name, 'O')
+                token.set_label(label_name, "O")
 
             top_spans = []
 
             for start_no, start_token in enumerate(sentence):
                 for stop_no, stop_token in enumerate(sentence):
-                    if stop_no < start_no: continue
+                    if stop_no < start_no:
+                        continue
 
                     prediction = predictions[s_no, start_no, stop_no].item()
                     confidence = confidences[s_no, start_no, stop_no].item()
@@ -307,7 +304,7 @@ class BiaffineTagger(flair.nn.Model):
                     top_spans.append((prediction, confidence, start_no, stop_no))
 
             # print(top_spans)
-            top_spans.sort(key=lambda x:x[1], reverse=True)
+            top_spans.sort(key=lambda x: x[1], reverse=True)
             # top_spans = [sorted(top_span, reverse=True, key=lambda x: x[1]) for top_span in top_spans]
             # print(top_spans)
 
@@ -317,25 +314,26 @@ class BiaffineTagger(flair.nn.Model):
                 if prediction != 0:
 
                     conflict = False
-                    for i in range(start_no, stop_no+1):
-                        if i in already_tagged: conflict = True
+                    for i in range(start_no, stop_no + 1):
+                        if i in already_tagged:
+                            conflict = True
 
                     if not conflict:
 
                         tag = self.tag_dictionary.get_item_for_index(prediction)
 
-                        span = ''
+                        span = ""
                         if start_no == stop_no:
 
-                            sentence[start_no].set_label(label_name, 'S-' + tag)
-                            span += ' ' + sentence[start_no].text
+                            sentence[start_no].set_label(label_name, "S-" + tag)
+                            span += " " + sentence[start_no].text
                             already_tagged.append(start_no)
 
                         else:
-                            for i in range(start_no, stop_no+1):
-                                label = 'B-' + tag if i == start_no else 'I-' + tag
+                            for i in range(start_no, stop_no + 1):
+                                label = "B-" + tag if i == start_no else "I-" + tag
                                 sentence[i].set_label(label_name, label)
-                                span += ' ' + sentence[i].text
+                                span += " " + sentence[i].text
                                 already_tagged.append(i)
 
                         # print(f'Added {span} as {tag} with confidence {confidence}')
@@ -349,7 +347,7 @@ class BiaffineTagger(flair.nn.Model):
         mini_batch_size=32,
         verbose: bool = False,
         label_name: Optional[str] = None,
-        return_loss = False,
+        return_loss=False,
         embedding_storage_mode="none",
     ):
         """
@@ -377,17 +375,11 @@ class BiaffineTagger(flair.nn.Model):
                 sentences = [sentences]
 
             # reverse sort all sequences by their length
-            rev_order_len_index = sorted(
-                range(len(sentences)), key=lambda k: len(sentences[k]), reverse=True
-            )
+            rev_order_len_index = sorted(range(len(sentences)), key=lambda k: len(sentences[k]), reverse=True)
 
-            reordered_sentences: List[Union[Sentence, str]] = [
-                sentences[index] for index in rev_order_len_index
-            ]
+            reordered_sentences: List[Union[Sentence, str]] = [sentences[index] for index in rev_order_len_index]
 
-            dataloader = DataLoader(
-                dataset=SentenceDataset(reordered_sentences), batch_size=mini_batch_size
-            )
+            dataloader = DataLoader(dataset=SentenceDataset(reordered_sentences), batch_size=mini_batch_size)
 
             # progress bar for verbosity
             if verbose:
@@ -456,11 +448,13 @@ class BiaffineTagger(flair.nn.Model):
         for batch in data_loader:
 
             # predict for batch
-            loss = self.predict(batch,
-                                embedding_storage_mode=embedding_storage_mode,
-                                mini_batch_size=mini_batch_size,
-                                label_name='predicted',
-                                return_loss=True)
+            loss = self.predict(
+                batch,
+                embedding_storage_mode=embedding_storage_mode,
+                mini_batch_size=mini_batch_size,
+                label_name="predicted",
+                return_loss=True,
+            )
             eval_loss += loss
             batch_no += 1
 
@@ -492,21 +486,21 @@ class BiaffineTagger(flair.nn.Model):
                 if out_path:
                     for token in sentence:
                         # check if in gold spans
-                        gold_tag = 'O'
+                        gold_tag = "O"
                         for span in gold_spans:
                             if token in span:
-                                gold_tag = 'B-' + span.tag if token == span[0] else 'I-' + span.tag
+                                gold_tag = "B-" + span.tag if token == span[0] else "I-" + span.tag
                         tags_gold.append(gold_tag)
 
-                        predicted_tag = 'O'
+                        predicted_tag = "O"
                         # check if in predicted spans
                         for span in predicted_spans:
                             if token in span:
-                                predicted_tag = 'B-' + span.tag if token == span[0] else 'I-' + span.tag
+                                predicted_tag = "B-" + span.tag if token == span[0] else "I-" + span.tag
                         tags_pred.append(predicted_tag)
 
-                        lines.append(f'{token.text} {gold_tag} {predicted_tag}\n')
-                    lines.append('\n')
+                        lines.append(f"{token.text} {gold_tag} {predicted_tag}\n")
+                    lines.append("\n")
 
                 y_true.append(tags_gold)
                 y_pred.append(tags_pred)
@@ -521,7 +515,7 @@ class BiaffineTagger(flair.nn.Model):
             "\nResults:"
             f"\n- F1-score (micro) {metric.micro_avg_f_score():.4f}"
             f"\n- F1-score (macro) {metric.macro_avg_f_score():.4f}"
-            '\n\nBy class:'
+            "\n\nBy class:"
         )
 
         for class_name in metric.get_classes():
@@ -589,15 +583,11 @@ class BiaffineTagger(flair.nn.Model):
         )
         all_embs = list()
         for sentence in sentences:
-            all_embs += [
-                emb for token in sentence for emb in token.get_each_embedding(names)
-            ]
+            all_embs += [emb for token in sentence for emb in token.get_each_embedding(names)]
             nb_padding_tokens = longest_token_sequence_in_batch - len(sentence)
 
             if nb_padding_tokens > 0:
-                t = pre_allocated_zero_tensor[
-                    : self.embeddings.embedding_length * nb_padding_tokens
-                    ]
+                t = pre_allocated_zero_tensor[: self.embeddings.embedding_length * nb_padding_tokens]
                 all_embs.append(t)
         sentence_tensor = torch.cat(all_embs).view(
             [
@@ -612,15 +602,13 @@ class BiaffineTagger(flair.nn.Model):
     def _filter_empty_sentences(sentences: List[Sentence]) -> List[Sentence]:
         filtered_sentences = [sentence for sentence in sentences if sentence.tokens]
         if len(sentences) != len(filtered_sentences):
-            log.warning(
-                f"Ignore {len(sentences) - len(filtered_sentences)} sentence(s) with no tokens."
-            )
+            log.warning(f"Ignore {len(sentences) - len(filtered_sentences)} sentence(s) with no tokens.")
         return filtered_sentences
 
     def _requires_span_F1_evaluation(self) -> bool:
         span_F1 = False
         for item in self.tag_dictionary.get_items():
-            if item.startswith('B-'):
+            if item.startswith("B-"):
                 span_F1 = True
         return span_F1
 
@@ -639,11 +627,13 @@ class BiaffineTagger(flair.nn.Model):
         for batch in data_loader:
 
             # predict for batch
-            loss = self.predict(batch,
-                                embedding_storage_mode=embedding_storage_mode,
-                                mini_batch_size=mini_batch_size,
-                                label_name='predicted',
-                                return_loss=True)
+            loss = self.predict(
+                batch,
+                embedding_storage_mode=embedding_storage_mode,
+                mini_batch_size=mini_batch_size,
+                label_name="predicted",
+                return_loss=True,
+            )
             eval_loss += loss
             batch_no += 1
 
@@ -675,21 +665,21 @@ class BiaffineTagger(flair.nn.Model):
                 if out_path:
                     for token in sentence:
                         # check if in gold spans
-                        gold_tag = 'O'
+                        gold_tag = "O"
                         for span in gold_spans:
                             if token in span:
-                                gold_tag = 'B-' + span.tag if token == span[0] else 'I-' + span.tag
+                                gold_tag = "B-" + span.tag if token == span[0] else "I-" + span.tag
                         tags_gold.append(gold_tag)
 
-                        predicted_tag = 'O'
+                        predicted_tag = "O"
                         # check if in predicted spans
                         for span in predicted_spans:
                             if token in span:
-                                predicted_tag = 'B-' + span.tag if token == span[0] else 'I-' + span.tag
+                                predicted_tag = "B-" + span.tag if token == span[0] else "I-" + span.tag
                         tags_pred.append(predicted_tag)
 
-                        lines.append(f'{token.text} {gold_tag} {predicted_tag}\n')
-                    lines.append('\n')
+                        lines.append(f"{token.text} {gold_tag} {predicted_tag}\n")
+                    lines.append("\n")
 
                 y_true.append(tags_gold)
                 y_pred.append(tags_pred)
@@ -704,7 +694,7 @@ class BiaffineTagger(flair.nn.Model):
             "\nResults:"
             f"\n- F1-score (micro) {metric.micro_avg_f_score():.4f}"
             f"\n- F1-score (macro) {metric.macro_avg_f_score():.4f}"
-            '\n\nBy class:'
+            "\n\nBy class:"
         )
 
         for class_name in metric.get_classes():
@@ -723,4 +713,4 @@ class BiaffineTagger(flair.nn.Model):
             detailed_results=detailed_result,
         )
 
-        return result, eval_loss
\ No newline at end of file
+        return result, eval_loss
