@@ -1,15 +1,15 @@
-from pathlib import Path
-
-import torch.nn as nn
-import torch
 import math
-from typing import Union, Tuple
-from typing import List
+from pathlib import Path
+from typing import List, Optional, Tuple, Union
 
+import torch
+import torch.nn as nn
+from torch import logsumexp
 from torch.optim import Optimizer
 
 import flair
 from flair.data import Dictionary
+from flair.nn.recurrent import create_recurrent_layer
 
 
 class LanguageModel(nn.Module):
@@ -23,8 +23,9 @@ class LanguageModel(nn.Module):
         nlayers: int,
         embedding_size: int = 100,
         nout=None,
-        document_delimiter: str = '\n',
+        document_delimiter: str = "\n",
         dropout=0.1,
+        recurrent_type="LSTM",
     ):
 
         super(LanguageModel, self).__init__()
@@ -41,16 +42,15 @@ class LanguageModel(nn.Module):
         self.drop = nn.Dropout(dropout)
         self.encoder = nn.Embedding(len(dictionary), embedding_size)
 
-        if nlayers == 1:
-            self.rnn = nn.LSTM(embedding_size, hidden_size, nlayers)
-        else:
-            self.rnn = nn.LSTM(embedding_size, hidden_size, nlayers, dropout=dropout)
-
+        self.rnn, self.state_count = create_recurrent_layer(
+            recurrent_type, embedding_size, hidden_size, nlayers, dropout
+        )
+        self.recurrent_type = recurrent_type
         self.hidden = None
 
         self.nout = nout
         if nout is not None:
-            self.proj = nn.Linear(hidden_size, nout)
+            self.proj: Optional[nn.Linear] = nn.Linear(hidden_size, nout)
             self.initialize(self.proj.weight)
             self.decoder = nn.Linear(nout, len(dictionary))
         else:
@@ -75,30 +75,32 @@ class LanguageModel(nn.Module):
         encoded = self.encoder(input)
         emb = self.drop(encoded)
 
-        self.rnn.flatten_parameters()
+        if hasattr(self.rnn, "flatten_parameters"):
+            self.rnn.flatten_parameters()
 
-        output, hidden = self.rnn(emb, hidden)
+        if len(hidden) == 1:
+            output, h = self.rnn(emb, hidden[0])
+            hidden = (h,)
+        else:
+            output, hidden = self.rnn(emb, hidden)
 
         if self.proj is not None:
             output = self.proj(output)
 
         output = self.drop(output)
 
-        decoded = self.decoder(
-            output.view(output.size(0) * output.size(1), output.size(2))
-        )
+        decoded = self.decoder(output)
 
         return (
-            decoded.view(output.size(0), output.size(1), decoded.size(1)),
+            decoded,
             output,
             hidden,
         )
 
     def init_hidden(self, bsz):
         weight = next(self.parameters()).detach()
-        return (
-            weight.new(self.nlayers, bsz, self.hidden_size).zero_().clone().detach(),
-            weight.new(self.nlayers, bsz, self.hidden_size).zero_().clone().detach(),
+        return tuple(
+            weight.new(self.nlayers, bsz, self.hidden_size).zero_().clone().detach() for _ in range(self.state_count)
         )
 
     def get_representation(
@@ -129,9 +131,7 @@ class LanguageModel(nn.Module):
             chunks.append([text[splice_begin:splice_end] for text in padded_strings])
             splice_begin = splice_end
 
-        chunks.append(
-            [text[splice_begin:longest_padded_str] for text in padded_strings]
-        )
+        chunks.append([text[splice_begin:longest_padded_str] for text in padded_strings])
         hidden = self.init_hidden(len(chunks[0]))
 
         padding_char_index = self.dictionary.get_idx_for_item(" ")
@@ -146,9 +146,7 @@ class LanguageModel(nn.Module):
                 char_indices += [padding_char_index] * (len_longest_chunk - len(string))
 
                 sequences_as_char_indices.append(char_indices)
-            t = torch.tensor(sequences_as_char_indices, dtype=torch.long).to(
-                device=flair.device, non_blocking=True
-            )
+            t = torch.tensor(sequences_as_char_indices, dtype=torch.long).to(device=flair.device, non_blocking=True)
             batches.append(t)
 
         output_parts = []
@@ -189,7 +187,7 @@ class LanguageModel(nn.Module):
 
         state = torch.load(str(model_file), map_location=flair.device)
 
-        document_delimiter = state["document_delimiter"] if "document_delimiter" in state else '\n'
+        document_delimiter = state.get("document_delimiter", "\n")
 
         model = LanguageModel(
             dictionary=state["dictionary"],
@@ -200,6 +198,7 @@ class LanguageModel(nn.Module):
             nout=state["nout"],
             document_delimiter=document_delimiter,
             dropout=state["dropout"],
+            recurrent_type=state.get("recurrent_type", "lstm"),
         )
         model.load_state_dict(state["state_dict"])
         model.eval()
@@ -214,11 +213,9 @@ class LanguageModel(nn.Module):
         epoch = state["epoch"] if "epoch" in state else None
         split = state["split"] if "split" in state else None
         loss = state["loss"] if "loss" in state else None
-        document_delimiter = state["document_delimiter"] if "document_delimiter" in state else '\n'
+        document_delimiter = state.get("document_delimiter", "\n")
 
-        optimizer_state_dict = (
-            state["optimizer_state_dict"] if "optimizer_state_dict" in state else None
-        )
+        optimizer_state_dict = state.get("optimizer_state_dict")
 
         model = LanguageModel(
             dictionary=state["dictionary"],
@@ -229,6 +226,7 @@ class LanguageModel(nn.Module):
             nout=state["nout"],
             document_delimiter=document_delimiter,
             dropout=state["dropout"],
+            recurrent_type=state.get("recurrent_type", "lstm"),
         )
         model.load_state_dict(state["state_dict"])
         model.eval()
@@ -243,7 +241,12 @@ class LanguageModel(nn.Module):
         }
 
     def save_checkpoint(
-        self, file: Union[Path, str], optimizer: Optimizer, epoch: int, split: int, loss: float
+        self,
+        file: Union[Path, str],
+        optimizer: Optimizer,
+        epoch: int,
+        split: int,
+        loss: float,
     ):
         model_state = {
             "state_dict": self.state_dict(),
@@ -259,6 +262,7 @@ class LanguageModel(nn.Module):
             "epoch": epoch,
             "split": split,
             "loss": loss,
+            "recurrent_type": self.recurrent_type,
         }
 
         torch.save(model_state, str(file), pickle_protocol=4)
@@ -274,6 +278,7 @@ class LanguageModel(nn.Module):
             "nout": self.nout,
             "document_delimiter": self.document_delimiter,
             "dropout": self.dropout,
+            "recurrent_type": self.recurrent_type,
         }
 
         torch.save(model_state, str(file), pickle_protocol=4)
@@ -302,22 +307,16 @@ class LanguageModel(nn.Module):
                 char_tensors = []
                 for character in prefix[:-1]:
                     char_tensors.append(
-                        torch.tensor(self.dictionary.get_idx_for_item(character))
-                        .unsqueeze(0)
-                        .unsqueeze(0)
+                        torch.tensor(self.dictionary.get_idx_for_item(character)).unsqueeze(0).unsqueeze(0)
                     )
 
                 input = torch.cat(char_tensors).to(flair.device)
 
                 prediction, _, hidden = self.forward(input, hidden)
 
-            input = (
-                torch.tensor(self.dictionary.get_idx_for_item(prefix[-1]))
-                .unsqueeze(0)
-                .unsqueeze(0)
-            )
+            input = torch.tensor(self.dictionary.get_idx_for_item(prefix[-1])).unsqueeze(0).unsqueeze(0)
 
-            log_prob = 0.0
+            log_prob = torch.zeros(1, device=flair.device)
 
             for i in range(number_of_characters):
 
@@ -342,11 +341,11 @@ class LanguageModel(nn.Module):
                 # try sampling multinomial distribution for next character
                 try:
                     word_idx = torch.multinomial(word_weights, 1)[0]
-                except:
+                except:  # noqa: E722 TODO: figure out exception type
                     word_idx = torch.tensor(0)
 
                 # print(word_idx)
-                prob = decoder_output[word_idx]
+                prob = decoder_output[word_idx] - logsumexp(decoder_output, dim=0)
                 log_prob += prob
 
                 input = word_idx.detach().unsqueeze(0).unsqueeze(0)
@@ -359,13 +358,13 @@ class LanguageModel(nn.Module):
 
             text = prefix + "".join(characters)
 
-            log_prob = log_prob.item()
-            log_prob /= len(characters)
+            log_prob_float = log_prob.item()
+            log_prob_float /= len(characters)
 
             if not self.is_forward_lm:
                 text = text[::-1]
 
-            return text, log_prob
+            return text, -log_prob_float
 
     def calculate_perplexity(self, text: str) -> float:
 
@@ -373,9 +372,7 @@ class LanguageModel(nn.Module):
             text = text[::-1]
 
         # input ids
-        input = torch.tensor(
-            [self.dictionary.get_idx_for_item(char) for char in text[:-1]]
-        ).unsqueeze(1)
+        input = torch.tensor([self.dictionary.get_idx_for_item(char) for char in text[:-1]]).unsqueeze(1)
         input = input.to(flair.device)
 
         # push list of character IDs through model
@@ -383,16 +380,12 @@ class LanguageModel(nn.Module):
         prediction, _, hidden = self.forward(input, hidden)
 
         # the target is always the next character
-        targets = torch.tensor(
-            [self.dictionary.get_idx_for_item(char) for char in text[1:]]
-        )
+        targets = torch.tensor([self.dictionary.get_idx_for_item(char) for char in text[1:]])
         targets = targets.to(flair.device)
 
         # use cross entropy loss to compare output of forward pass with targets
         cross_entroy_loss = torch.nn.CrossEntropyLoss()
-        loss = cross_entroy_loss(
-            prediction.view(-1, len(self.dictionary)), targets
-        ).item()
+        loss = cross_entroy_loss(prediction.view(-1, len(self.dictionary)), targets).item()
 
         # exponentiate cross-entropy loss to calculate perplexity
         perplexity = math.exp(loss)
@@ -400,11 +393,12 @@ class LanguageModel(nn.Module):
         return perplexity
 
     def __getstate__(self):
+        # "document_delimiter" property may be missing in some older pre-trained models
+        self.document_delimiter = getattr(self, "document_delimiter", "\n")
 
         # serialize the language models and the constructor arguments (but nothing else)
         model_state = {
             "state_dict": self.state_dict(),
-
             "dictionary": self.dictionary,
             "is_forward_lm": self.is_forward_lm,
             "hidden_size": self.hidden_size,
@@ -413,6 +407,7 @@ class LanguageModel(nn.Module):
             "nout": self.nout,
             "document_delimiter": self.document_delimiter,
             "dropout": self.dropout,
+            "recurrent_type": self.recurrent_type,
         }
 
         return model_state
@@ -424,17 +419,18 @@ class LanguageModel(nn.Module):
 
             # re-initialize language model with constructor arguments
             language_model = LanguageModel(
-                dictionary=d['dictionary'],
-                is_forward_lm=d['is_forward_lm'],
-                hidden_size=d['hidden_size'],
-                nlayers=d['nlayers'],
-                embedding_size=d['embedding_size'],
-                nout=d['nout'],
-                document_delimiter=d['document_delimiter'],
-                dropout=d['dropout'],
+                dictionary=d["dictionary"],
+                is_forward_lm=d["is_forward_lm"],
+                hidden_size=d["hidden_size"],
+                nlayers=d["nlayers"],
+                embedding_size=d["embedding_size"],
+                nout=d["nout"],
+                document_delimiter=d["document_delimiter"],
+                dropout=d["dropout"],
+                recurrent_type=d.get("recurrent_type", "lstm"),
             )
 
-            language_model.load_state_dict(d['state_dict'])
+            language_model.load_state_dict(d["state_dict"])
 
             # copy over state dictionary to self
             for key in language_model.__dict__.keys():
@@ -445,6 +441,10 @@ class LanguageModel(nn.Module):
             self.eval()
 
         else:
+            if "recurrent_type" not in d:
+                d["recurrent_type"] = "lstm"
+            if "state_count" not in d:
+                d["state_count"] = 2
             super().__setstate__(d)
 
     def _apply(self, fn):
@@ -465,12 +465,9 @@ class LanguageModel(nn.Module):
                         param_names = ["weight_ih_l{}{}", "weight_hh_l{}{}"]
                         if child_module.__dict__["bias"]:
                             param_names += ["bias_ih_l{}{}", "bias_hh_l{}{}"]
-                        param_names = [
-                            x.format(layer, suffix) for x in param_names
-                        ]
+                        param_names = [x.format(layer, suffix) for x in param_names]
                         _flat_weights_names.extend(param_names)
 
-                setattr(child_module, "_flat_weights_names",
-                        _flat_weights_names)
+                setattr(child_module, "_flat_weights_names", _flat_weights_names)
 
-            child_module._apply(fn)
\ No newline at end of file
+            child_module._apply(fn)
