@@ -1,12 +1,11 @@
 import logging
 from abc import ABC, abstractmethod
-from typing import List, Callable, Optional
+from typing import Any, Callable, List, Union
 
-from more_itertools import stagger
-from segtok.segmenter import split_single, split_multi
+from segtok.segmenter import split_multi, split_single
 from segtok.tokenizer import split_contractions, word_tokenizer
 
-from flair.data import Sentence, Token, Tokenizer
+from flair.data import Sentence, Tokenizer
 
 log = logging.getLogger("flair")
 
@@ -35,62 +34,42 @@ class SpacyTokenizer(Tokenizer):
         elif isinstance(model, str):
             self.model: Language = spacy.load(model)
         else:
-            raise AssertionError(f"Unexpected type of parameter model. Please provide a loaded "
-                                 f"spacy model or the name of the model to load.")
+            raise AssertionError(
+                "Unexpected type of parameter model. Please provide a loaded "
+                "spacy model or the name of the model to load."
+            )
 
-    def tokenize(self, text: str) -> List[Token]:
+    def tokenize(self, text: str) -> List[str]:
         from spacy.tokens.doc import Doc
-        from spacy.tokens.token import Token as SpacyToken
 
         doc: Doc = self.model.make_doc(text)
-        previous_token = None
-        tokens: List[Token] = []
+        words: List[str] = []
         for word in doc:
-            word: SpacyToken = word
             if len(word.text.strip()) == 0:
                 continue
-
-            token = Token(
-                text=word.text, start_position=word.idx, whitespace_after=True
-            )
-            tokens.append(token)
-
-            if (previous_token is not None) and (
-                    token.start_pos == previous_token.start_pos + len(previous_token.text)
-            ):
-                previous_token.whitespace_after = False
-
-            previous_token = token
-
-        return tokens
+            words.append(word.text)
+        return words
 
     @property
     def name(self) -> str:
-        return (
-                self.__class__.__name__
-                + "_"
-                + self.model.meta["name"]
-                + "_"
-                + self.model.meta["version"]
-        )
+        return self.__class__.__name__ + "_" + self.model.meta["name"] + "_" + self.model.meta["version"]
 
 
 class SegtokTokenizer(Tokenizer):
     """
-        Tokenizer using segtok, a third party library dedicated to rules-based Indo-European languages.
+    Tokenizer using segtok, a third party library dedicated to rules-based Indo-European languages.
 
-        For further details see: https://github.com/fnl/segtok
+    For further details see: https://github.com/fnl/segtok
     """
 
     def __init__(self):
         super(SegtokTokenizer, self).__init__()
 
-    def tokenize(self, text: str) -> List[Token]:
+    def tokenize(self, text: str) -> List[str]:
         return SegtokTokenizer.run_tokenize(text)
 
     @staticmethod
-    def run_tokenize(text: str) -> List[Token]:
-        tokens: List[Token] = []
+    def run_tokenize(text: str) -> List[str]:
         words: List[str] = []
 
         sentences = split_single(text)
@@ -100,62 +79,29 @@ class SegtokTokenizer(Tokenizer):
 
         words = list(filter(None, words))
 
-        # determine offsets for whitespace_after field
-        index = text.index
-        current_offset = 0
-        previous_word_offset = -1
-        previous_token = None
-        for word in words:
-            try:
-                word_offset = index(word, current_offset)
-                start_position = word_offset
-            except:
-                word_offset = previous_word_offset + 1
-                start_position = (
-                    current_offset + 1 if current_offset > 0 else current_offset
-                )
-
-            if word:
-                token = Token(
-                    text=word, start_position=start_position, whitespace_after=True
-                )
-                tokens.append(token)
-
-            if (previous_token is not None) and word_offset - 1 == previous_word_offset:
-                previous_token.whitespace_after = False
-
-            current_offset = word_offset + len(word)
-            previous_word_offset = current_offset - 1
-            previous_token = token
-
-        return tokens
+        return words
 
 
 class SpaceTokenizer(Tokenizer):
     """
-        Tokenizer based on space character only.
+    Tokenizer based on space character only.
     """
 
     def __init__(self):
         super(SpaceTokenizer, self).__init__()
 
-    def tokenize(self, text: str) -> List[Token]:
+    def tokenize(self, text: str) -> List[str]:
         return SpaceTokenizer.run_tokenize(text)
 
     @staticmethod
-    def run_tokenize(text: str) -> List[Token]:
-        tokens: List[Token] = []
+    def run_tokenize(text: str) -> List[str]:
+        tokens: List[str] = []
         word = ""
         index = -1
         for index, char in enumerate(text):
             if char == " ":
                 if len(word) > 0:
-                    start_position = index - len(word)
-                    tokens.append(
-                        Token(
-                            text=word, start_position=start_position, whitespace_after=True
-                        )
-                    )
+                    tokens.append(word)
 
                 word = ""
             else:
@@ -163,21 +109,18 @@ class SpaceTokenizer(Tokenizer):
         # increment for last token in sentence if not followed by whitespace
         index += 1
         if len(word) > 0:
-            start_position = index - len(word)
-            tokens.append(
-                Token(text=word, start_position=start_position, whitespace_after=False)
-            )
+            tokens.append(word)
 
         return tokens
 
 
 class JapaneseTokenizer(Tokenizer):
     """
-        Tokenizer using konoha, a third party library which supports
-        multiple Japanese tokenizer such as MeCab, Janome and SudachiPy.
+    Tokenizer using konoha, a third party library which supports
+    multiple Japanese tokenizer such as MeCab, Janome and SudachiPy.
 
-        For further details see:
-            https://github.com/himkt/konoha
+    For further details see:
+        https://github.com/himkt/konoha
     """
 
     def __init__(self, tokenizer: str, sudachi_mode: str = "A"):
@@ -207,8 +150,7 @@ class JapaneseTokenizer(Tokenizer):
         self.sentence_tokenizer = konoha.SentenceTokenizer()
         self.word_tokenizer = konoha.WordTokenizer(tokenizer, mode=sudachi_mode)
 
-    def tokenize(self, text: str) -> List[Token]:
-        tokens: List[Token] = []
+    def tokenize(self, text: str) -> List[str]:
         words: List[str] = []
 
         sentences = self.sentence_tokenizer.tokenize(text)
@@ -216,54 +158,23 @@ class JapaneseTokenizer(Tokenizer):
             konoha_tokens = self.word_tokenizer.tokenize(sentence)
             words.extend(list(map(str, konoha_tokens)))
 
-        # determine offsets for whitespace_after field
-        index = text.index
-        current_offset = 0
-        previous_word_offset = -1
-        previous_token = None
-        for word in words:
-            try:
-                word_offset = index(word, current_offset)
-                start_position = word_offset
-            except:
-                word_offset = previous_word_offset + 1
-                start_position = (
-                    current_offset + 1 if current_offset > 0 else current_offset
-                )
-
-            token = Token(
-                text=word, start_position=start_position, whitespace_after=True
-            )
-            tokens.append(token)
-
-            if (previous_token is not None) and word_offset - 1 == previous_word_offset:
-                previous_token.whitespace_after = False
-
-            current_offset = word_offset + len(word)
-            previous_word_offset = current_offset - 1
-            previous_token = token
-
-        return tokens
+        return words
 
     @property
     def name(self) -> str:
-        return (
-                self.__class__.__name__
-                + "_"
-                + self.tokenizer
-        )
+        return self.__class__.__name__ + "_" + self.tokenizer
 
 
 class TokenizerWrapper(Tokenizer):
     """
-        Helper class to wrap tokenizer functions to the class-based tokenizer interface.
+    Helper class to wrap tokenizer functions to the class-based tokenizer interface.
     """
 
-    def __init__(self, tokenizer_func: Callable[[str], List[Token]]):
+    def __init__(self, tokenizer_func: Callable[[str], List[str]]):
         super(TokenizerWrapper, self).__init__()
         self.tokenizer_func = tokenizer_func
 
-    def tokenize(self, text: str) -> List[Token]:
+    def tokenize(self, text: str) -> List[str]:
         return self.tokenizer_func(text)
 
     @property
@@ -273,13 +184,13 @@ class TokenizerWrapper(Tokenizer):
 
 class SciSpacyTokenizer(Tokenizer):
     """
-        Implementation of :class:`Tokenizer` which uses the en_core_sci_sm Spacy model
-        extended by special heuristics to consider characters such as "(", ")" "-" as
-        additional token separators. The latter distinguishs this implementation from
-        :class:`SpacyTokenizer`.
+    Implementation of :class:`Tokenizer` which uses the en_core_sci_sm Spacy model
+    extended by special heuristics to consider characters such as "(", ")" "-" as
+    additional token separators. The latter distinguishs this implementation from
+    :class:`SpacyTokenizer`.
 
-        Note, you if you want to use the "normal" SciSpacy tokenization just use
-        :class:`SpacyTokenizer`.
+    Note, you if you want to use the "normal" SciSpacy tokenization just use
+    :class:`SpacyTokenizer`.
     """
 
     def __init__(self):
@@ -301,81 +212,56 @@ class SciSpacyTokenizer(Tokenizer):
 
         def combined_rule_prefixes() -> List[str]:
             """Helper function that returns the prefix pattern for the tokenizer.
-               It is a helper function to accommodate spacy tests that only test
-               prefixes.
+            It is a helper function to accommodate spacy tests that only test
+            prefixes.
             """
             prefix_punct = char_classes.PUNCT.replace("|", " ")
 
             prefixes = (
-                    ["§", "%", "=", r"\+"]
-                    + char_classes.split_chars(prefix_punct)
-                    + char_classes.LIST_ELLIPSES
-                    + char_classes.LIST_QUOTES
-                    + char_classes.LIST_CURRENCY
-                    + char_classes.LIST_ICONS
+                ["§", "%", "=", r"\+"]
+                + char_classes.split_chars(prefix_punct)
+                + char_classes.LIST_ELLIPSES
+                + char_classes.LIST_QUOTES
+                + char_classes.LIST_CURRENCY
+                + char_classes.LIST_ICONS
             )
             return prefixes
 
         infixes = (
-                char_classes.LIST_ELLIPSES
-                + char_classes.LIST_ICONS
-                + [
-                    r"×",  # added this special x character to tokenize it separately
-                    r"[\(\)\[\]\{\}]",  # want to split at every bracket
-                    r"/",  # want to split at every slash
-                    r"(?<=[0-9])[+\-\*^](?=[0-9-])",
-                    r"(?<=[{al}])\.(?=[{au}])".format(
-                        al=char_classes.ALPHA_LOWER, au=char_classes.ALPHA_UPPER
-                    ),
-                    r"(?<=[{a}]),(?=[{a}])".format(a=char_classes.ALPHA),
-                    r'(?<=[{a}])[?";:=,.]*(?:{h})(?=[{a}])'.format(
-                        a=char_classes.ALPHA, h=char_classes.HYPHENS
-                    ),
-                    r"(?<=[{a}0-9])[:<>=/](?=[{a}])".format(a=char_classes.ALPHA),
-                ]
+            char_classes.LIST_ELLIPSES
+            + char_classes.LIST_ICONS
+            + [
+                r"×",  # added this special x character to tokenize it separately
+                r"[\(\)\[\]\{\}]",  # want to split at every bracket
+                r"/",  # want to split at every slash
+                r"(?<=[0-9])[+\-\*^](?=[0-9-])",
+                r"(?<=[{al}])\.(?=[{au}])".format(al=char_classes.ALPHA_LOWER, au=char_classes.ALPHA_UPPER),
+                r"(?<=[{a}]),(?=[{a}])".format(a=char_classes.ALPHA),
+                r'(?<=[{a}])[?";:=,.]*(?:{h})(?=[{a}])'.format(a=char_classes.ALPHA, h=char_classes.HYPHENS),
+                r"(?<=[{a}0-9])[:<>=/](?=[{a}])".format(a=char_classes.ALPHA),
+            ]
         )
 
         prefix_re = spacy.util.compile_prefix_regex(combined_rule_prefixes())
         infix_re = spacy.util.compile_infix_regex(infixes)
 
         self.model = spacy.load(
-            "en_core_sci_sm", disable=["tagger", "ner", "parser", "textcat", "lemmatizer"]
+            "en_core_sci_sm",
+            disable=["tagger", "ner", "parser", "textcat", "lemmatizer"],
         )
         self.model.tokenizer.prefix_search = prefix_re.search
         self.model.tokenizer.infix_finditer = infix_re.finditer
 
-    def tokenize(self, text: str) -> List[Token]:
-        from spacy.tokens.token import Token as SpacyToken
-
+    def tokenize(self, text: str) -> List[str]:
         sentence = self.model(text)
-
-        previous_token = None
-        tokens: List[Token] = []
+        words: List[str] = []
         for word in sentence:
-            word: SpacyToken = word
-            token = Token(
-                text=word.text, start_position=word.idx, whitespace_after=True
-            )
-            tokens.append(token)
-
-            if (previous_token is not None) and (
-                    token.start_pos == previous_token.start_pos + len(previous_token.text)
-            ) and (not word.text[0].isspace()):
-                previous_token.whitespace_after = False
-
-            previous_token = token
-
-        return tokens
+            words.append(word.text)
+        return words
 
     @property
     def name(self) -> str:
-        return (
-                self.__class__.__name__
-                + "_"
-                + self.model.meta["name"]
-                + "_"
-                + self.model.meta["version"]
-        )
+        return self.__class__.__name__ + "_" + self.model.meta["name"] + "_" + self.model.meta["version"]
 
 
 class SentenceSplitter(ABC):
@@ -410,9 +296,9 @@ class SentenceSplitter(ABC):
 
 class SegtokSentenceSplitter(SentenceSplitter):
     """
-        Implementation of :class:`SentenceSplitter` using the SegTok library.
+    Implementation of :class:`SentenceSplitter` using the SegTok library.
 
-        For further details see: https://github.com/fnl/segtok
+    For further details see: https://github.com/fnl/segtok
     """
 
     def __init__(self, tokenizer: Tokenizer = SegtokTokenizer()):
@@ -420,31 +306,27 @@ class SegtokSentenceSplitter(SentenceSplitter):
         self._tokenizer = tokenizer
 
     def split(self, text: str) -> List[Sentence]:
-        plain_sentences: List[str] = list(split_multi(text))
-
-        try:
-            sentence_offset: Optional[int] = text.index(plain_sentences[0])
-        except ValueError as error:
-            raise AssertionError(f"Can't find the sentence offset for sentence {repr(plain_sentences[0])} "
-                                 f"from the text's starting position") from error
+        plain_sentences: List[str] = split_multi(text)
+        sentence_offset = 0
 
         sentences: List[Sentence] = []
-        for sentence, next_sentence in stagger(plain_sentences, offsets=(0, 1), longest=True):
-
+        for sentence in plain_sentences:
+            try:
+                sentence_offset = text.index(sentence, sentence_offset)
+            except ValueError as error:
+                raise AssertionError(
+                    f"Can't find the sentence offset for sentence {repr(sentence)} "
+                    f"starting from position {repr(sentence_offset)}"
+                ) from error
             sentences.append(
                 Sentence(
                     text=sentence,
                     use_tokenizer=self._tokenizer,
-                    start_position=sentence_offset
+                    start_position=sentence_offset,
                 )
             )
 
-            offset: int = sentence_offset + len(sentence)
-            try:
-                sentence_offset = text.index(next_sentence, offset) if next_sentence is not None else None
-            except ValueError as error:
-                raise AssertionError(f"Can't find the sentence offset for sentence {repr(sentence)} "
-                                     f"starting from position {repr(offset)}") from error
+            sentence_offset += len(sentence)
 
         return sentences
 
@@ -469,7 +351,7 @@ class SpacySentenceSplitter(SentenceSplitter):
     :param tokenizer Custom tokenizer to use (default :class:`SpacyTokenizer`)
     """
 
-    def __init__(self, model: str, tokenizer: Tokenizer = None):
+    def __init__(self, model: Union[Any, str], tokenizer: Tokenizer = None):
         super(SpacySentenceSplitter, self).__init__()
 
         try:
@@ -483,11 +365,12 @@ class SpacySentenceSplitter(SentenceSplitter):
 
         if isinstance(model, Language):
             self.model: Language = model
-        elif isinstance(model, str):
-            self.model: Language = spacy.load(model)
+        else:
+            assert isinstance(model, str)
+            self.model = spacy.load(model)
 
         if tokenizer is None:
-            self._tokenizer = SpacyTokenizer("en_core_sci_sm")
+            self._tokenizer: Tokenizer = SpacyTokenizer("en_core_sci_sm")
         else:
             self._tokenizer = tokenizer
 
@@ -498,7 +381,7 @@ class SpacySentenceSplitter(SentenceSplitter):
             Sentence(
                 text=str(spacy_sent),
                 use_tokenizer=self._tokenizer,
-                start_position=spacy_sent.start_char
+                start_position=spacy_sent.start_char,
             )
             for spacy_sent in document.sents
             if len(str(spacy_sent)) > 0
@@ -517,13 +400,13 @@ class SpacySentenceSplitter(SentenceSplitter):
     @property
     def name(self) -> str:
         return (
-                self.__class__.__name__
-                + "_"
-                + self.model.meta["name"]
-                + "_"
-                + self.model.meta["version"]
-                + "_"
-                + self._tokenizer.name
+            self.__class__.__name__
+            + "_"
+            + self.model.meta["name"]
+            + "_"
+            + self.model.meta["version"]
+            + "_"
+            + self._tokenizer.name
         )
 
 
@@ -562,7 +445,7 @@ class TagSentenceSplitter(SentenceSplitter):
                 Sentence(
                     text=sentence,
                     use_tokenizer=self._tokenizer,
-                    start_position=last_offset
+                    start_position=last_offset,
                 )
             ]
 
@@ -580,13 +463,7 @@ class TagSentenceSplitter(SentenceSplitter):
 
     @property
     def name(self) -> str:
-        return (
-                self.__class__.__name__
-                + "_"
-                + self.tag
-                + "_"
-                + self._tokenizer.name
-        )
+        return self.__class__.__name__ + "_" + self.tag + "_" + self._tokenizer.name
 
 
 class NewlineSentenceSplitter(TagSentenceSplitter):
@@ -600,11 +477,7 @@ class NewlineSentenceSplitter(TagSentenceSplitter):
 
     @property
     def name(self) -> str:
-        return (
-                self.__class__.__name__
-                + "_"
-                + self._tokenizer.name
-        )
+        return self.__class__.__name__ + "_" + self._tokenizer.name
 
 
 class NoSentenceSplitter(SentenceSplitter):
@@ -617,13 +490,7 @@ class NoSentenceSplitter(SentenceSplitter):
         self._tokenizer = tokenizer
 
     def split(self, text: str) -> List[Sentence]:
-        return [
-            Sentence(
-                text=text,
-                use_tokenizer=self._tokenizer,
-                start_position=0
-            )
-        ]
+        return [Sentence(text=text, use_tokenizer=self._tokenizer, start_position=0)]
 
     @property
     def tokenizer(self) -> Tokenizer:
@@ -635,8 +502,4 @@ class NoSentenceSplitter(SentenceSplitter):
 
     @property
     def name(self) -> str:
-        return (
-                self.__class__.__name__
-                + "_"
-                + self._tokenizer.name
-        )
+        return self.__class__.__name__ + "_" + self._tokenizer.name
