@@ -1,12 +1,20 @@
-import time, datetime
+import datetime
+import logging
+import math
 import random
 import sys
+import time
 from pathlib import Path
-from typing import Union
+from typing import Iterable, Type, Union
 
+import torch
 from torch import cuda
-from torch.utils.data import Dataset, DataLoader
+from torch.optim import AdamW, Optimizer
+from torch.optim.lr_scheduler import ReduceLROnPlateau
 from torch.optim.sgd import SGD
+from torch.utils.data import DataLoader, Dataset
+
+from flair.optim import SGDW, ReduceLRWDOnPlateau
 
 try:
     from apex import amp
@@ -16,7 +24,6 @@ except ImportError:
 import flair
 from flair.data import Dictionary
 from flair.models import LanguageModel
-from flair.optim import *
 from flair.training_utils import add_file_handler
 
 log = logging.getLogger("flair")
@@ -31,14 +38,12 @@ class TextDataset(Dataset):
         forward: bool = True,
         split_on_char: bool = True,
         random_case_flip: bool = True,
-        document_delimiter: str = '\n',
+        document_delimiter: str = "\n",
         shuffle: bool = True,
     ):
-        if type(path) is str:
-            path = Path(path)
+        path = Path(path)
         assert path.exists()
 
-        self.files = None
         self.path = path
         self.dictionary = dictionary
         self.split_on_char = split_on_char
@@ -56,23 +61,25 @@ class TextDataset(Dataset):
     def __len__(self):
         return len(self.files)
 
-    def __getitem__(self, index=0) -> torch.tensor:
+    def __getitem__(self, index=0) -> torch.Tensor:
         """Tokenizes a text file on character basis."""
         if type(self.files[index]) is str:
             self.files[index] = Path(self.files[index])
         assert self.files[index].exists()
 
         with self.files[index].open("r", encoding="utf-8") as fin:
-            lines = (doc + self.document_delimiter for doc in fin.read().split(self.document_delimiter) if doc)
+            text_lines: Iterable[str] = (
+                doc + self.document_delimiter for doc in fin.read().split(self.document_delimiter) if doc
+            )
             if self.random_case_flip:
-                lines = map(self.random_casechange, lines)
-            lines = list(map(list if self.split_on_char else str.split, lines))
+                text_lines = map(self.random_casechange, text_lines)
+            lines = list(map(list if self.split_on_char else str.split, text_lines))  # type: ignore # noqa: E501
 
         log.info(f"read text file with {len(lines)} lines")
 
         if self.shuffle:
             random.shuffle(lines)
-            log.info(f"shuffled")
+            log.info("shuffled")
 
         if self.expand_vocab:
             for chars in lines:
@@ -80,8 +87,8 @@ class TextDataset(Dataset):
                     self.dictionary.add_item(char)
 
         ids = torch.tensor(
-            [self.dictionary.get_idx_for_item(char) for chars in lines for char in chars], 
-            dtype=torch.long
+            [self.dictionary.get_idx_for_item(char) for chars in lines for char in chars],
+            dtype=torch.long,
         )
         if not self.forward:
             ids = ids.flip(0)
@@ -105,7 +112,7 @@ class TextCorpus(object):
         forward: bool = True,
         character_level: bool = True,
         random_case_flip: bool = True,
-        document_delimiter: str = '\n',
+        document_delimiter: str = "\n",
     ):
         self.dictionary: Dictionary = dictionary
         self.forward = forward
@@ -113,8 +120,7 @@ class TextCorpus(object):
         self.random_case_flip = random_case_flip
         self.document_delimiter: str = document_delimiter
 
-        if type(path) == str:
-            path = Path(path)
+        path = Path(path)
 
         self.train = TextDataset(
             path / "train",
@@ -127,7 +133,8 @@ class TextCorpus(object):
             shuffle=True,
         )
 
-        # TextDataset returns a list. valid and test are only one file, so return the first element
+        # TextDataset returns a list. valid and test are only one file,
+        # so return the first element
         self.valid = TextDataset(
             path / "valid.txt",
             dictionary,
@@ -155,7 +162,7 @@ class LanguageModelTrainer:
         self,
         model: LanguageModel,
         corpus: TextCorpus,
-        optimizer: Optimizer = SGD,
+        optimizer: Type[Optimizer] = SGD,
         test_mode: bool = False,
         epoch: int = 0,
         split: int = 0,
@@ -163,7 +170,7 @@ class LanguageModelTrainer:
         optimizer_state: dict = None,
     ):
         self.model: LanguageModel = model
-        self.optimizer: Optimizer = optimizer
+        self.optimizer: Type[Optimizer] = optimizer
         self.corpus: TextCorpus = corpus
         self.test_mode: bool = test_mode
 
@@ -197,15 +204,13 @@ class LanguageModelTrainer:
                 raise RuntimeError("Apex currently only supports Python 3. Aborting.")
             if amp is None:
                 raise RuntimeError(
-                    "Failed to import apex. Please install apex from https://www.github.com/nvidia/apex "
+                    "Failed to import apex. Please install apex from "
+                    "https://www.github.com/nvidia/apex "
                     "to enable mixed-precision training."
                 )
 
         # cast string to Path
-        if type(base_path) is str:
-            base_path = Path(base_path)
-
-        add_file_handler(log, base_path / "training.log")
+        base_path = Path(base_path)
 
         number_of_splits: int = len(self.corpus.train)
 
@@ -214,7 +219,8 @@ class LanguageModelTrainer:
         # error message if the validation dataset is too small
         if val_data.size(0) == 1:
             raise RuntimeError(
-                f"ERROR: Your validation dataset is too small. For your mini_batch_size, the data needs to "
+                f"ERROR: Your validation dataset is too small. For your "
+                f"mini_batch_size, the data needs to "
                 f"consist of at least {mini_batch_size * 2} characters!"
             )
 
@@ -223,38 +229,32 @@ class LanguageModelTrainer:
         savefile = base_path / "best-lm.pt"
 
         try:
+            log_handler = add_file_handler(log, base_path / "training.log")
+
             best_val_loss = self.loss
-            optimizer = self.optimizer(
-                self.model.parameters(), lr=learning_rate, **kwargs
-            )
+            kwargs["lr"] = learning_rate
+            optimizer = self.optimizer(self.model.parameters(), **kwargs)
             if self.optimizer_state is not None:
                 optimizer.load_state_dict(self.optimizer_state)
 
             if isinstance(optimizer, (AdamW, SGDW)):
-                scheduler: ReduceLRWDOnPlateau = ReduceLRWDOnPlateau(
+                scheduler: ReduceLROnPlateau = ReduceLRWDOnPlateau(
                     optimizer, verbose=True, factor=anneal_factor, patience=patience
                 )
             else:
-                scheduler: ReduceLROnPlateau = ReduceLROnPlateau(
-                    optimizer, verbose=True, factor=anneal_factor, patience=patience
-                )
+                scheduler = ReduceLROnPlateau(optimizer, verbose=True, factor=anneal_factor, patience=patience)
 
             if use_amp:
-                self.model, optimizer = amp.initialize(
-                    self.model, optimizer, opt_level=amp_opt_level
-                )
+                self.model, optimizer = amp.initialize(self.model, optimizer, opt_level=amp_opt_level)
 
-            training_generator = DataLoader(
-                self.corpus.train, shuffle=False, num_workers=num_workers
-            )
+            training_generator = DataLoader(self.corpus.train, shuffle=False, num_workers=num_workers)
 
             for epoch in range(self.epoch, max_epochs):
                 epoch_start_time = time.time()
-                # Shuffle training files randomly after serially iterating through corpus one
+                # Shuffle training files randomly after serially iterating
+                # through corpus one
                 if epoch > 0:
-                    training_generator = DataLoader(
-                        self.corpus.train, shuffle=True, num_workers=num_workers
-                    )
+                    training_generator = DataLoader(self.corpus.train, shuffle=True, num_workers=num_workers)
                     self.model.save_checkpoint(
                         base_path / f"epoch_{epoch}.pt",
                         optimizer,
@@ -263,10 +263,9 @@ class LanguageModelTrainer:
                         best_val_loss,
                     )
 
-                # iterate through training data, starting at self.split (for checkpointing)
-                for curr_split, train_slice in enumerate(
-                    training_generator, self.split
-                ):
+                # iterate through training data, starting at
+                # self.split (for checkpointing)
+                for curr_split, train_slice in enumerate(training_generator, self.split):
 
                     if sequence_length < grow_to_sequence_length:
                         sequence_length += 1
@@ -277,10 +276,7 @@ class LanguageModelTrainer:
                     curr_split += 1
                     train_data = self._batchify(train_slice.flatten(), mini_batch_size)
 
-                    log.info(
-                        "Split %d" % curr_split
-                        + "\t - ({:%H:%M:%S})".format(datetime.datetime.now())
-                    )
+                    log.info("Split %d" % curr_split + "\t - ({:%H:%M:%S})".format(datetime.datetime.now()))
 
                     for group in optimizer.param_groups:
                         learning_rate = group["lr"]
@@ -294,19 +290,14 @@ class LanguageModelTrainer:
                     # not really sure what this does
                     ntokens = len(self.corpus.dictionary)
 
-                    total_loss = 0
+                    total_loss = torch.zeros(1, device=flair.device)
                     start_time = time.time()
 
-                    for batch, i in enumerate(
-                        range(0, train_data.size(0) - 1, sequence_length)
-                    ):
+                    for batch, i in enumerate(range(0, train_data.size(0) - 1, sequence_length)):
                         data, targets = self._get_batch(train_data, i, sequence_length)
 
                         if not data.is_cuda and cuda.is_available():
-                            log.info(
-                                "Batch %d is not on CUDA, training will be very slow"
-                                % (batch)
-                            )
+                            log.info("Batch %d is not on CUDA, training will be very slow" % (batch))
                             raise Exception("data isnt on cuda")
 
                         self.model.zero_grad()
@@ -324,15 +315,18 @@ class LanguageModelTrainer:
                         else:
                             loss.backward()
 
-                        # `clip_grad_norm` helps prevent the exploding gradient problem in RNNs / LSTMs.
+                        # `clip_grad_norm` helps prevent the exploding gradient
+                        # problem in RNNs / LSTMs.
                         torch.nn.utils.clip_grad_norm_(self.model.parameters(), clip)
 
                         optimizer.step()
 
                         total_loss += loss.data
 
-                        # We detach the hidden state from how it was previously produced.
-                        # If we didn't, the model would try backpropagating all the way to start of the dataset.
+                        # We detach the hidden state from how it was
+                        # previously produced.
+                        # If we didn't, the model would try backpropagating
+                        # all the way to start of the dataset.
                         hidden = self._repackage_hidden(hidden)
 
                         # explicitly remove loss to clear up memory
@@ -342,32 +336,27 @@ class LanguageModelTrainer:
                             cur_loss = total_loss.item() / self.log_interval
                             elapsed = time.time() - start_time
                             log.info(
-                                "| split {:3d} /{:3d} | {:5d}/{:5d} batches | ms/batch {:5.2f} | "
-                                "loss {:5.2f} | ppl {:8.2f}".format(
-                                    curr_split,
-                                    number_of_splits,
-                                    batch,
-                                    len(train_data) // sequence_length,
-                                    elapsed * 1000 / self.log_interval,
-                                    cur_loss,
-                                    math.exp(cur_loss),
-                                )
+                                f"| split {curr_split:3d}/{number_of_splits:3d} | {batch:5d}/{len(train_data) // sequence_length:5d} batches "
+                                f"| ms/batch {elapsed * 1000 / self.log_interval:5.2f} | loss {cur_loss:5.4f} | ppl {math.exp(cur_loss):5.4f}"
                             )
-                            total_loss = 0
+                            total_loss = torch.zeros(1, device=flair.device)
                             start_time = time.time()
 
-                    log.info(
-                        "%d seconds for train split %d"
-                        % (time.time() - split_start_time, curr_split)
-                    )
-
-                    ###############################################################################
+                    ##########################################################
                     self.model.eval()
 
                     val_loss = self.evaluate(val_data, mini_batch_size, sequence_length)
+
+                    # Save the model if the validation loss is the best we've
+                    # seen so far.
+                    if val_loss < best_val_loss:
+                        self.model.save(savefile)
+                        best_val_loss = val_loss
+                        log.info("best split so far")
+
                     scheduler.step(val_loss)
 
-                    log.info("best loss so far {:5.2f}".format(best_val_loss))
+                    log.info(f"best loss so far {best_val_loss:5.8f}")
 
                     log.info(self.model.generate_text())
 
@@ -380,28 +369,15 @@ class LanguageModelTrainer:
                             best_val_loss,
                         )
 
-                    # Save the model if the validation loss is the best we've seen so far.
-                    if val_loss < best_val_loss:
-                        self.model.best_score = best_val_loss
-                        self.model.save(savefile)
-                        best_val_loss = val_loss
-
-                    ###############################################################################
+                    ##########################################################
                     # print info
-                    ###############################################################################
+                    ##########################################################
                     log.info("-" * 89)
 
                     summary = (
-                        "| end of split {:3d} /{:3d} | epoch {:3d} | time: {:5.2f}s | valid loss {:5.2f} | "
-                        "valid ppl {:8.2f} | learning rate {:3.4f}".format(
-                            curr_split,
-                            number_of_splits,
-                            epoch + 1,
-                            (time.time() - split_start_time),
-                            val_loss,
-                            math.exp(val_loss),
-                            learning_rate,
-                        )
+                        f"| end of split {curr_split:3d} /{number_of_splits:3d} | epoch {epoch + 1:3d} | time: "
+                        f"{(time.time() - split_start_time):5.2f}s | valid loss {val_loss:5.4f} | valid ppl "
+                        f"{math.exp(val_loss):5.4f} | learning rate {learning_rate:3.4f}"
                     )
 
                     with open(loss_txt, "a") as myfile:
@@ -409,12 +385,17 @@ class LanguageModelTrainer:
 
                     log.info(summary)
                     log.info("-" * 89)
+                    log.info("%d seconds for train split %d" % (time.time() - split_start_time, curr_split))
 
                 log.info("Epoch time: %.2f" % (time.time() - epoch_start_time))
 
         except KeyboardInterrupt:
             log.info("-" * 89)
             log.info("Exiting from training early")
+        finally:
+            if log_handler is not None:
+                log_handler.close()
+                log.removeHandler(log_handler)
 
         ###############################################################################
         # final testing
@@ -422,9 +403,7 @@ class LanguageModelTrainer:
         test_data = self._batchify(self.corpus.test, mini_batch_size)
         test_loss = self.evaluate(test_data, mini_batch_size, sequence_length)
 
-        summary = "TEST: valid loss {:5.2f} | valid ppl {:8.2f}".format(
-            test_loss, math.exp(test_loss)
-        )
+        summary = f"TEST: valid loss {test_loss:5.4f} | valid ppl {math.exp(test_loss):8.4f}"
         with open(loss_txt, "a") as myfile:
             myfile.write("%s\n" % summary)
 
@@ -463,8 +442,8 @@ class LanguageModelTrainer:
     def _get_batch(source, i, sequence_length):
         seq_len = min(sequence_length, len(source) - 1 - i)
 
-        data = source[i : i + seq_len].clone().detach()
-        target = source[i + 1 : i + 1 + seq_len].view(-1).clone().detach()
+        data = source[i : i + seq_len]
+        target = source[i + 1 : i + 1 + seq_len].view(-1)
 
         data = data.to(flair.device)
         target = target.to(flair.device)
@@ -474,11 +453,13 @@ class LanguageModelTrainer:
     @staticmethod
     def _repackage_hidden(h):
         """Wraps hidden states in new tensors, to detach them from their history."""
-        return tuple(v.clone().detach() for v in h)
+        return tuple(v.detach() for v in h)
 
     @staticmethod
     def load_checkpoint(
-        checkpoint_file: Union[str, Path], corpus: TextCorpus, optimizer: Optimizer = SGD
+        checkpoint_file: Union[str, Path],
+        corpus: TextCorpus,
+        optimizer: Type[Optimizer] = SGD,
     ):
         if type(checkpoint_file) is str:
             checkpoint_file = Path(checkpoint_file)
